<?php
namespace Teknisa\CDP\Factories;

use Doctrine\ORM\EntityManager; 
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class OpdigitalFactory {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager    = $entityManager;
        $this->connection       = $entityManager->getConnection();
    }
    
    public function xdqVerifProdPiloto($row){
        try {
            $sql = "SELECT O.IDPRODPILOTO 
                    FROM ORDPRODUCAO O 
                    WHERE O.CDFILIAL     = :P_CDFILIAL   AND
                        O.NRORDPROD    = :P_NRORDPROD    AND
                        O.NRSUBORDPROD = :P_NRSUBORDPROD AND
                        O.CDPRODUTO    = :P_CDPRODUTO    AND
                        O.IDPRODPILOTO = 'S'";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_NRORDPROD'       => $row['NRORDPROD'],
                'P_NRSUBORDPROD'    => $row['NRSUBORDPROD'],
                'P_CDPRODUTO'       => $row['CDPRODUTO']
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqItemRetiEst($row){
        try {
            $sql = "SELECT
                       ITEMRETIEST.CDFILIAL      ,
                       ITEMRETIEST.NRRETEST      ,
                       ITEMRETIEST.NRSEQRETPROD  ,
                       ITEMRETIEST.CDPRODUTO     ,
                       ITEMRETIEST.QTPRERETPROD  ,
                       ITEMRETIEST.DTRETPROD     ,
                       ITEMRETIEST.CDTIPORETI    ,
                       RETIRADAESTO.CDETAPAPROD  ,
                       NVL(TIPORETI.IDTIPOMOVI, 'A') IDTIPOMOVI,
                       NVL(INSUMO.IDCOMPATI, 'N') IDCOMPATI
                    FROM
                       RETIRADAESTO, ITEMRETIEST, TIPORETI, ORDPRODUCAO, 
                       (SELECT INSUMETAPA.CDFILIAL, INSUMETAPA.CDPRODINSUMO, INSUMETAPA.IDCOMPATI
                         FROM ORDPRODUCAO, PROCFABRICA, INSUMETAPA
                       WHERE ORDPRODUCAO.CDFILIAL      = :P_CDFILIAL
                         AND ORDPRODUCAO.NRORDPROD    = :P_NRORDPROD
                         AND ORDPRODUCAO.NRSUBORDPROD = :P_NRSUBORDPROD
                         AND ORDPRODUCAO.CDFILIAL = PROCFABRICA.CDFILIAL
                         AND ORDPRODUCAO.CDPRODUTO = PROCFABRICA.CDPRODUTO
                         AND ORDPRODUCAO.CDPROCFABRIC  = PROCFABRICA.CDPROCFABRIC
                         AND PROCFABRICA.CDFILIAL      = INSUMETAPA.CDFILIAL(+)
                         AND PROCFABRICA.CDPRODUTO     = INSUMETAPA.CDPRODUTO(+)
                         AND PROCFABRICA.CDPROCFABRIC  = INSUMETAPA.CDPROCFABRIC(+)) INSUMO
                    WHERE
                       ITEMRETIEST.CDFILIAL      = :P_CDFILIAL                AND
                       RETIRADAESTO.NRORDPROD    = :P_NRORDPROD               AND
                       RETIRADAESTO.NRSUBORDPROD = :P_NRSUBORDPROD            AND
                       RETIRADAESTO.CDFILIAL     = ORDPRODUCAO.CDFILIAL       AND
                       RETIRADAESTO.NRORDPROD    = ORDPRODUCAO.NRORDPROD      AND
                       RETIRADAESTO.NRSUBORDPROD = ORDPRODUCAO.NRSUBORDPROD   AND
                       ITEMRETIEST.CDFILIAL      = RETIRADAESTO.CDFILIAL      AND
                       ITEMRETIEST.NRRETEST      = RETIRADAESTO.NRRETEST      AND
                       ITEMRETIEST.CDTIPORETI    = TIPORETI.CDTIPORETI(+)     AND
                       ITEMRETIEST.CDFILIAL    = INSUMO.CDFILIAL(+)           AND
                       ITEMRETIEST.CDPRODUTO = INSUMO.CDPRODINSUMO(+)
                    ORDER BY
                       ITEMRETIEST.CDPRODUTO";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_NRORDPROD'       => $row['NRORDPROD'],
                'P_NRSUBORDPROD'    => $row['NRSUBORDPROD']
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqOrdProducao($row){
        try {
            $sql = "SELECT
                       O.CDPRODUTO    ,
                       O.CDPROCFABRIC ,
                       PF.IDVERIFCOMPAT,
                       O.QTAPRODPREOP ,
                       O.NRLOTEESTQ   ,
                       O.NRSUBLOTE    ,
                       O.CDPLANPRODOP,
                       O.NRORDPROD,
                       O.NRSUBORDPROD,
                       O.NRPEDIDO,
                       O.NRITEMPED,
                       DECODE(P.IDVDANALISE, '1', 'Meses: ' || NVL(P.NRDIASVALANA, 0),
                                                  'Fórmula: ' || NVL(P.DSFORMULAVDANA, 'Vazia')) AS FORMULA
                    FROM
                       ORDPRODUCAO O, PRODUTO P, PROCFABRICA PF
                    WHERE
                       O.CDFILIAL     = :P_CDFILIAL      AND
                       O.NRORDPROD    = :P_NRORDPROD     AND
                       O.NRSUBORDPROD = :P_NRSUBORDPROD  AND
                       O.CDPRODUTO    = P.CDPRODUTO      AND
                       O.CDFILIAL     = PF.CDFILIAL      AND
                       O.CDPRODUTO    = PF.CDPRODUTO     AND
                       O.CDPROCFABRIC = PF.CDPROCFABRIC";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_NRORDPROD'       => $row['NRORDPROD'],
                'P_NRSUBORDPROD'    => $row['NRSUBORDPROD']
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqRetiradaEsto($row, $nrretest){
        try {
            $sql = "SELECT
                       NRLANCESTQ
                    FROM
                      RETIRADAESTO
                    WHERE
                       CDFILIALLAN = :P_CDFILIAL AND
                       NRRETEST    = :P_NRRETEST";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_NRRETEST'        => $nrretest
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmDel_LotItRetiEst($cdfilial, $nrretest, $nrseqretprod, $cdproduto){
        try {
            
            $sql = "DELETE FROM LOTITRETIEST
                    WHERE CDFILIAL     = :P_CDFILIAL
                      AND NRRETEST     = :P_NRRETEST
                      AND NRSEQRETPROD = :P_NRSEQRETPROD
                      AND CDPRODUTO    = :P_CDPRODUTO";
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRRETEST'        => $nrretest,
                'P_NRSEQRETPROD'    => $nrseqretprod,
                'P_CDPRODUTO'       => $cdproduto
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqInsumetapa($row, $cdetapaprod, $cdproduto){
        try {
            $sql = "SELECT
                       IDLOTEPRETAP,
                       IDUTAPRREQET,
                       NVL( IDSEMIACETAP, 'N') AS IDSEMIACETAP,
                       IDCOMPATI,
                       NVL(VRFATOPADR, 1) VRFATOPADR
                    FROM
                       INSUMETAPA
                    WHERE
                       CDFILIAL     = :P_CDFILIAL     AND
                       CDPRODUTO    = :P_CDPRODUTO    AND
                       CDPROCFABRIC = :P_CDPROCFABRIC AND
                       CDETAPAPROD  = :P_CDETAPAPROD  AND
                       CDPRODINSUMO = :P_CDPRODINSUMO";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_CDPRODUTO'       => $row['CDPRODUTO'],
                'P_CDPROCFABRIC'    => $row['CDPROCFABRIC'],
                'P_CDETAPAPROD'     => $cdetapaprod,
                'P_CDPRODINSUMO'    => $cdproduto,
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqVerif_Insumos($row, $cdetapaprod, $cdproduto, $cdfilial){
        try {
            
            $sql = "SELECT
                       I.CDPRODUTO    ,
                       I.CDPROCFABRIC ,
                       I.CDETAPAPROD  ,
                       I.CDPRODINSUMO ,
                       I.IDUTAPRREQET ,
                       I.QTINSUMETAPA ,
                       I.QTRENDINSU   ,
                       I.IDSEMIACETAP ,
                       I.IDCOMPATI    ,
                       I.CDTIPOINSU,
                       P.NMPRODUTO,
                       P.SGUNIDADE,
                       I.IDLOTEPRETAP,
                       I.IDLOTEUNICO,
                       I.DSFASE
                    FROM
                       INSUMETAPA I,
                       ETAFABRICOP E,
                       PRODUTO P
                    WHERE
                       I.CDFILIAL     = E.CDFILIAL                           AND
                       I.CDPRODUTO    = :P_CDPRODUTO                         AND
                       I.CDPROCFABRIC = :P_CDPROCFABRIC                      AND
                       I.CDETAPAPROD  = NVL(:P_CDETAPAPROD,  I.CDETAPAPROD)  AND
                       I.CDPRODINSUMO = NVL(:P_CDPRODINSUMO, I.CDPRODINSUMO) AND
                       E.CDFILIAL     = :P_CDFILIAL                          AND
                       E.NRORDPROD    = :P_NRORDPROD                         AND
                       E.NRSUBORDPROD = :P_NRSUBORDPROD                      AND
                       E.CDETAPAPROD  = I.CDETAPAPROD                        AND
                       P.CDPRODUTO    = I.CDPRODINSUMO";
            $params = array(
                'P_CDPRODUTO'       => $row['CDPRODUTO'],
                'P_CDPROCFABRIC'    => $row['CDPROCFABRIC'],
                'P_CDETAPAPROD'     => $cdetapaprod,
                'P_CDPRODINSUMO'    => $cdproduto,
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $row['NRORDPROD'],
                'P_NRSUBORDPROD'    => $row['NRSUBORDPROD']
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqEstMatPrima($row, $cdproduto, $idtpretirada, $qtretiprod){
        
        // var_dump($row, $cdproduto, $idtpretirada, $qtretiprod);die;
        
        try {
            $sql = "SELECT
                       E.CDFILIAL      ,
                       E.CDPRODUTO     ,
                       E.CDALMOXARIFE  ,
                       E.CDLOCALESTOQ  ,
                       E.NRLOTEESTQ    ,
                       E.NRSUBLOTE     ,
                       E.QTTOTALESTOQ  ,
                       E.VRAPROVPROD   ,
                       E.VRCUSTMEDATU  ,
                       E.DTVALILOTE    ,
                       E.DTFABRPROD    ,
                       E.CDLOCALINSPR  ,
                       E.IDITEMPEDTERC ,
                       E.VRFATOCONV    ,
                       DECODE(E.CDALMOXARIFE, E.CDALMOXREUT, '1', '2') ORDEM --Almox de insumos reutilizáveis primeiro
                    FROM
                       (SELECT P.CDFILIAL,
                           P.CDPRODUTO,
                           P.CDALMOXARIFE, PROD.CDALMOXREUT,
                           P.CDLOCALESTOQ, PROD.CDLOCALREUT,
                           P.NRLOTEESTQ,
                           P.NRSUBLOTE,
                           P.QTTOTALESTOQ,
                           L.VRAPROVPROD,
                           L.DTVALILOTE,
                           L.DTFABRPROD,
                           L.DTENTRLOTE,
                           PROD.CDALMOXPRACA,
                           PROD.CDLOCALPRACA,
                           PROD.VRCUSTMEDATU, L.CDORDENALOTE,PROD.CDLOCALINSPR,
                           'N' AS IDITEMPEDTERC,
                           L.VRFATOCONV
                    FROM LOTE L, PRODFILI PROD, POSESTATU P, PARAMFILIAL PA, PARAMAN PR, PRODFILI P2
                    WHERE (PR.CDFILIAL = PROD.CDFILIAL)         AND
                          (PROD.CDFILIAL     = P.CDFILIAL)      AND
                          (PROD.CDFILIAL     = :P_CDFILIAL)     AND
                          (P2.CDFILIAL       = :P_CDFILIAL)    AND
                          (P2.CDPRODUTO      = :P_CDPRODINSUMO) AND
                          (P.CDPRODUTO       = :P_CDPRODINSUMO) AND
                          (PROD.CDPRODUTO =   DECODE( NVL( PR.IDUTLOCESTPRACA, 'N'), 'N' , :P_CDPRODINSUMO, :P_CDPRODUTO)) AND
                    
                          ((P.CDALMOXARIFE = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('' ,PROD.CDALMOXINSPRPIL),NVL('', PROD.CDALMOXINSPR)),' ')  AND
                            P.CDLOCALESTOQ = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('', PROD.CDLOCALINSPRPIL),NVL('' ,PROD.CDLOCALINSPR)),' ')) OR
                    
                            --Insumos reutilizáveis devem ser exibidos primeiro e somente se existir quantidade total em estoque
                            (P.CDALMOXARIFE = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('' ,PROD.CDALMOXINSPRPIL),NVL('', PROD.CDALMOXREUT)),' ') AND
                             P.CDLOCALESTOQ = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('', PROD.CDLOCALINSPRPIL),NVL('' ,PROD.CDLOCALREUT)),' ') AND
                             P.QTTOTALESTOQ >= :P_QTRETIPROD) OR
                    
                           (P.CDALMOXARIFE = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('' ,PROD.CDALMOXINSPRPIL),NVL('',PROD.CDALMOXPRSEMACA)),' ')  AND
                            P.CDLOCALESTOQ  = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('', PROD.CDLOCALINSPRPIL),NVL('',PROD.CDLOCALPRSEMACA)),' '))
                    
                           )AND
                          (P.CDFILIAL        = L.CDFILIAL)      AND
                          (P.NRLOTEESTQ      = L.NRLOTEESTQ)    AND
                          (P.NRSUBLOTE       = L.NRSUBLOTE)     AND
                          (P.CDPRODUTO       = L.CDPRODUTO)     AND
                          (P.QTTOTALESTOQ > 0)                  AND
                          (PA.CDFILIAL       = P.CDFILIAL)      AND
                          (PA.IDUTILALMOX    = 'S')             AND
                          ((L.CDPLANPRODLT = :P_CDPLANPRODLT) OR ('T' = 'T')) AND
                          (((TRUNC(NVL(L.DTVALILOTE, SYSDATE)) >= TRUNC(SYSDATE)) AND (P2.IDUTCTRLLOTE = 'S')) OR
                          (PR.IDLOTEVENC = 'S') OR
                          (P2.IDUTCTRLLOTE <> 'S'))
                    UNION
                    
                    SELECT P.CDFILIAL,
                           P.CDPRODUTO,
                           P.CDALMOXARIFE, ' ' AS CDALMOXREUT,
                           P.CDLOCALESTOQ, ' ' AS CDLOCALREUT,
                           P.NRLOTEESTQ,
                           P.NRSUBLOTE,
                           P.QTTOTALESTOQ,
                           L.VRAPROVPROD,
                           L.DTVALILOTE,
                           L.DTFABRPROD,
                           L.DTENTRLOTE,
                           ' ' AS CDALMOXPRACA,
                           ' ' AS CDLOCALPRACA,
                           0   AS VRCUSTMEDATU, L.CDORDENALOTE, '',
                           'N' AS IDITEMPEDTERC,
                           L.VRFATOCONV
                    FROM LOTE L, POSESTATU P, PARAMFILIAL PA, PRODFILI PROD, PARAMAN PR, PRODFILI P2
                    WHERE (P.CDFILIAL        = L.CDFILIAL)      AND
                          (P.CDFILIAL        = :P_CDFILIAL)     AND
                          (P.CDPRODUTO       = :P_CDPRODINSUMO) AND
                          (P2.CDFILIAL        = :P_CDFILIAL)     AND
                          (P2.CDPRODUTO       = :P_CDPRODINSUMO) AND
                          (P.NRLOTEESTQ      = L.NRLOTEESTQ)    AND
                          (P.NRSUBLOTE       = L.NRSUBLOTE)     AND
                    
                          (P.CDPRODUTO       = L.CDPRODUTO)     AND
                          (P.QTTOTALESTOQ > 0)                  AND
                          (PA.CDFILIAL       = P.CDFILIAL)      AND
                          (PA.IDUTILALMOX    <> 'S')            AND
                          ((L.CDPLANPRODLT = :P_CDPLANPRODLT) OR ('T' = 'T')) AND
                          (PR.CDFILIAL       = P.CDFILIAL)      AND
                          (PROD.CDFILIAL = P.CDFILIAL)          AND
                          (PROD.CDPRODUTO =   DECODE( NVL( PR.IDUTLOCESTPRACA, 'N'), 'N' , :P_CDPRODINSUMO, :P_CDPRODUTO)) AND
                          (((TRUNC(NVL(L.DTVALILOTE, SYSDATE)) >= TRUNC(SYSDATE)) AND (P2.IDUTCTRLLOTE = 'S')) OR
                          (PR.IDLOTEVENC = 'S') OR
                          (P2.IDUTCTRLLOTE <> 'S'))
                    
                          UNION
                    
                            SELECT PO.CDFILIAL    ,
                                   PO.CDPRODUTO   ,
                                   PO.CDALMOXARIFE, ' ' AS CDALMOXREUT,
                                   PO.CDLOCALESTOQ, ' ' AS CDLOCALREUT,
                                   PO.NRLOTEESTQ  ,
                                   PO.NRSUBLOTE   ,
                                   PO.QTTOTALESTOQ,
                                   LO.VRAPROVPROD ,
                                   LO.DTVALILOTE  ,
                                   LO.DTFABRPROD,
                                   LO.DTENTRLOTE  ,
                                   ' ' AS CDALMOXPRACA,
                                   ' ' AS CDLOCALPRACA,
                                   0 AS VRCUSTMEDATU,
                                   LO.CDORDENALOTE,'',
                                   IDITEMPEDTERC,
                                   LO.VRFATOCONV
                              FROM AGENTETERCECLI AC,
                                   ORDPRODUCAO    OP,
                                   POSESTATU      PO,
                                   ITEMPEDIDO     IP,
                                   PEDIDO         PE,
                                   LOTE           LO,
                                   PRODFILI PROD,
                                   PARAMAN PR
                             WHERE (PO.CDFILIAL       = :P_CDFILIAL    )
                               AND (PO.CDPRODUTO      = :P_CDPRODINSUMO)
                               AND (OP.NRORDPROD      = :P_NRORDPROD   )
                               AND (OP.NRSUBORDPROD   = :P_NRSUBORDPROD)
                               AND (OP.CDPRODUTO      = :P_CDPRODUTO   )
                               AND (PO.CDFILIAL       = AC.CDFILIAL    )
                               AND (PO.CDALMOXARIFE   = AC.CDALMOXTERCCLI)
                               AND (PO.CDLOCALESTOQ   = AC.CDLOCALTERCCLI)
                               AND (PO.CDFILIAL       = LO.CDFILIAL    )
                               AND (PO.NRLOTEESTQ     = LO.NRLOTEESTQ  )
                               AND (PO.NRSUBLOTE      = LO.NRSUBLOTE   )
                               AND (PO.CDPRODUTO      = LO.CDPRODUTO   )
                               AND (IP.CDFILIAL       = OP.CDFILIAL    )
                               AND (IP.NRPEDIDO       = OP.NRPEDIDO    )
                               AND (IP.NRITEMPED      = OP.NRITEMPED   )
                               AND (PE.CDFILIAL       = IP.CDFILIAL    )
                               AND (PE.NRPEDIDO       = IP.NRPEDIDO    )
                               AND (PE.CDCLIENTE      = AC.CDCLIENTE   )
                               AND (PE.IDPEDINDUSTRIA = 'S'            )
                               AND (IP.IDITEMPEDTERC  = 'S'            )
                               AND (PO.QTTOTALESTOQ   > 0              )
                               AND ((LO.CDPLANPRODLT  = :P_CDPLANPRODLT) OR ('T' = 'T'))
                               AND (PROD.CDFILIAL     = :P_CDFILIAL)
                               AND (PR.CDFILIAL       = PROD.CDFILIAL)
                               AND (PROD.CDPRODUTO =   DECODE( NVL( PR.IDUTLOCESTPRACA, 'N'), 'N' , :P_CDPRODINSUMO, :P_CDPRODUTO))
                               AND (((TRUNC(LO.DTVALILOTE) >= TRUNC(SYSDATE)) AND (PROD.IDUTCTRLLOTE = 'S'))
                                OR (PR.IDLOTEVENC = 'S')
                                OR (PROD.IDUTCTRLLOTE <> 'S'))
                    
                          ) E
                    ORDER BY ORDEM, IDITEMPEDTERC DESC,
                       (CASE WHEN :P_IDTPRETIRADA ='FI' THEN E.DTENTRLOTE
                             WHEN :P_IDTPRETIRADA = 'FE' THEN E.DTVALILOTE END),
                       DECODE(:P_IDTPRETIRADA, 'LI', E.DTENTRLOTE) DESC, E.DTENTRLOTE, DTFABRPROD, NRLOTEESTQ";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_CDPRODINSUMO'    => $cdproduto,
                'P_CDPRODUTO'       => $row['CDPRODUTO'],
                'P_IDPRODPILOTO'    => $row['IDPRODPILOTO'],
                'P_NRORDPROD'       => $row['NRORDPROD'],
                'P_NRSUBORDPROD'    => $row['NRSUBORDPROD'],
                'P_CDPROCFABRIC'    => $row['CDPROCFABRIC'],
                'P_CDPLANPRODLT'    => '',
                'P_IDTPRETIRADA'    => $idtpretirada,
                'P_QTRETIPROD'      => $qtretiprod
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqEstMatPrimaFilter($row, $cdproduto, $idtpretirada, $qtretiprod, $nrloteestq, $nrsublote){
        try {
            $sql = "SELECT * FROM (SELECT
                       E.CDFILIAL      ,
                       E.CDPRODUTO     ,
                       E.CDALMOXARIFE  ,
                       E.CDLOCALESTOQ  ,
                       E.NRLOTEESTQ    ,
                       E.NRSUBLOTE     ,
                       E.QTTOTALESTOQ  ,
                       E.VRAPROVPROD   ,
                       E.VRCUSTMEDATU  ,
                       E.DTVALILOTE    ,
                       E.DTFABRPROD    ,
                       E.CDLOCALINSPR  ,
                       E.IDITEMPEDTERC ,
                       E.VRFATOCONV    ,
                       DECODE(E.CDALMOXARIFE, E.CDALMOXREUT, '1', '2') ORDEM --Almox de insumos reutilizáveis primeiro
                    FROM
                       (SELECT P.CDFILIAL,
                           P.CDPRODUTO,
                           P.CDALMOXARIFE, PROD.CDALMOXREUT,
                           P.CDLOCALESTOQ, PROD.CDLOCALREUT,
                           P.NRLOTEESTQ,
                           P.NRSUBLOTE,
                           P.QTTOTALESTOQ,
                           L.VRAPROVPROD,
                           L.DTVALILOTE,
                           L.DTFABRPROD,
                           L.DTENTRLOTE,
                           PROD.CDALMOXPRACA,
                           PROD.CDLOCALPRACA,
                           PROD.VRCUSTMEDATU, L.CDORDENALOTE,PROD.CDLOCALINSPR,
                           'N' AS IDITEMPEDTERC,
                           L.VRFATOCONV
                    FROM LOTE L, PRODFILI PROD, POSESTATU P, PARAMFILIAL PA, PARAMAN PR, PRODFILI P2
                    WHERE (PR.CDFILIAL = PROD.CDFILIAL)         AND
                          (PROD.CDFILIAL     = P.CDFILIAL)      AND
                          (PROD.CDFILIAL     = :P_CDFILIAL)     AND
                          (P2.CDFILIAL       = :P_CDFILIAL)    AND
                          (P2.CDPRODUTO      = :P_CDPRODINSUMO) AND
                          (P.CDPRODUTO       = :P_CDPRODINSUMO) AND
                          (PROD.CDPRODUTO =   DECODE( NVL( PR.IDUTLOCESTPRACA, 'N'), 'N' , :P_CDPRODINSUMO, :P_CDPRODUTO)) AND
                    
                          ((P.CDALMOXARIFE = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('' ,PROD.CDALMOXINSPRPIL),NVL('', PROD.CDALMOXINSPR)),' ')  AND
                            P.CDLOCALESTOQ = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('', PROD.CDLOCALINSPRPIL),NVL('' ,PROD.CDLOCALINSPR)),' ')) OR
                    
                            --Insumos reutilizáveis devem ser exibidos primeiro e somente se existir quantidade total em estoque
                            (P.CDALMOXARIFE = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('' ,PROD.CDALMOXINSPRPIL),NVL('', PROD.CDALMOXREUT)),' ') AND
                             P.CDLOCALESTOQ = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('', PROD.CDLOCALINSPRPIL),NVL('' ,PROD.CDLOCALREUT)),' ') AND
                             P.QTTOTALESTOQ >= :P_QTRETIPROD) OR
                    
                           (P.CDALMOXARIFE = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('' ,PROD.CDALMOXINSPRPIL),NVL('',PROD.CDALMOXPRSEMACA)),' ')  AND
                            P.CDLOCALESTOQ  = NVL(DECODE(:P_IDPRODPILOTO,'S',NVL('', PROD.CDLOCALINSPRPIL),NVL('',PROD.CDLOCALPRSEMACA)),' '))
                    
                           )AND
                          (P.CDFILIAL        = L.CDFILIAL)      AND
                          (P.NRLOTEESTQ      = L.NRLOTEESTQ)    AND
                          (P.NRSUBLOTE       = L.NRSUBLOTE)     AND
                          (P.CDPRODUTO       = L.CDPRODUTO)     AND
                          (P.QTTOTALESTOQ > 0)                  AND
                          (PA.CDFILIAL       = P.CDFILIAL)      AND
                          (PA.IDUTILALMOX    = 'S')             AND
                          ((L.CDPLANPRODLT = :P_CDPLANPRODLT) OR ('T' = 'T')) AND
                          (((TRUNC(NVL(L.DTVALILOTE, SYSDATE)) >= TRUNC(SYSDATE)) AND (P2.IDUTCTRLLOTE = 'S')) OR
                          (PR.IDLOTEVENC = 'S') OR
                          (P2.IDUTCTRLLOTE <> 'S'))
                    UNION
                    
                    SELECT P.CDFILIAL,
                           P.CDPRODUTO,
                           P.CDALMOXARIFE, ' ' AS CDALMOXREUT,
                           P.CDLOCALESTOQ, ' ' AS CDLOCALREUT,
                           P.NRLOTEESTQ,
                           P.NRSUBLOTE,
                           P.QTTOTALESTOQ,
                           L.VRAPROVPROD,
                           L.DTVALILOTE,
                           L.DTFABRPROD,
                           L.DTENTRLOTE,
                           ' ' AS CDALMOXPRACA,
                           ' ' AS CDLOCALPRACA,
                           0   AS VRCUSTMEDATU, L.CDORDENALOTE, '',
                           'N' AS IDITEMPEDTERC,
                           L.VRFATOCONV
                    FROM LOTE L, POSESTATU P, PARAMFILIAL PA, PRODFILI PROD, PARAMAN PR, PRODFILI P2
                    WHERE (P.CDFILIAL        = L.CDFILIAL)      AND
                          (P.CDFILIAL        = :P_CDFILIAL)     AND
                          (P.CDPRODUTO       = :P_CDPRODINSUMO) AND
                          (P2.CDFILIAL        = :P_CDFILIAL)     AND
                          (P2.CDPRODUTO       = :P_CDPRODINSUMO) AND
                          (P.NRLOTEESTQ      = L.NRLOTEESTQ)    AND
                          (P.NRSUBLOTE       = L.NRSUBLOTE)     AND
                    
                          (P.CDPRODUTO       = L.CDPRODUTO)     AND
                          (P.QTTOTALESTOQ > 0)                  AND
                          (PA.CDFILIAL       = P.CDFILIAL)      AND
                          (PA.IDUTILALMOX    <> 'S')            AND
                          ((L.CDPLANPRODLT = :P_CDPLANPRODLT) OR ('T' = 'T')) AND
                          (PR.CDFILIAL       = P.CDFILIAL)      AND
                          (PROD.CDFILIAL = P.CDFILIAL)          AND
                          (PROD.CDPRODUTO =   DECODE( NVL( PR.IDUTLOCESTPRACA, 'N'), 'N' , :P_CDPRODINSUMO, :P_CDPRODUTO)) AND
                          (((TRUNC(NVL(L.DTVALILOTE, SYSDATE)) >= TRUNC(SYSDATE)) AND (P2.IDUTCTRLLOTE = 'S')) OR
                          (PR.IDLOTEVENC = 'S') OR
                          (P2.IDUTCTRLLOTE <> 'S'))
                    
                          UNION
                    
                            SELECT PO.CDFILIAL    ,
                                   PO.CDPRODUTO   ,
                                   PO.CDALMOXARIFE, ' ' AS CDALMOXREUT,
                                   PO.CDLOCALESTOQ, ' ' AS CDLOCALREUT,
                                   PO.NRLOTEESTQ  ,
                                   PO.NRSUBLOTE   ,
                                   PO.QTTOTALESTOQ,
                                   LO.VRAPROVPROD ,
                                   LO.DTVALILOTE  ,
                                   LO.DTFABRPROD,
                                   LO.DTENTRLOTE  ,
                                   ' ' AS CDALMOXPRACA,
                                   ' ' AS CDLOCALPRACA,
                                   0 AS VRCUSTMEDATU,
                                   LO.CDORDENALOTE,'',
                                   IDITEMPEDTERC,
                                   LO.VRFATOCONV
                              FROM AGENTETERCECLI AC,
                                   ORDPRODUCAO    OP,
                                   POSESTATU      PO,
                                   ITEMPEDIDO     IP,
                                   PEDIDO         PE,
                                   LOTE           LO,
                                   PRODFILI PROD,
                                   PARAMAN PR
                             WHERE (PO.CDFILIAL       = :P_CDFILIAL    )
                               AND (PO.CDPRODUTO      = :P_CDPRODINSUMO)
                               AND (OP.NRORDPROD      = :P_NRORDPROD   )
                               AND (OP.NRSUBORDPROD   = :P_NRSUBORDPROD)
                               AND (OP.CDPRODUTO      = :P_CDPRODUTO   )
                               AND (PO.CDFILIAL       = AC.CDFILIAL    )
                               AND (PO.CDALMOXARIFE   = AC.CDALMOXTERCCLI)
                               AND (PO.CDLOCALESTOQ   = AC.CDLOCALTERCCLI)
                               AND (PO.CDFILIAL       = LO.CDFILIAL    )
                               AND (PO.NRLOTEESTQ     = LO.NRLOTEESTQ  )
                               AND (PO.NRSUBLOTE      = LO.NRSUBLOTE   )
                               AND (PO.CDPRODUTO      = LO.CDPRODUTO   )
                               AND (IP.CDFILIAL       = OP.CDFILIAL    )
                               AND (IP.NRPEDIDO       = OP.NRPEDIDO    )
                               AND (IP.NRITEMPED      = OP.NRITEMPED   )
                               AND (PE.CDFILIAL       = IP.CDFILIAL    )
                               AND (PE.NRPEDIDO       = IP.NRPEDIDO    )
                               AND (PE.CDCLIENTE      = AC.CDCLIENTE   )
                               AND (PE.IDPEDINDUSTRIA = 'S'            )
                               AND (IP.IDITEMPEDTERC  = 'S'            )
                               AND (PO.QTTOTALESTOQ   > 0              )
                               AND ((LO.CDPLANPRODLT  = :P_CDPLANPRODLT) OR ('T' = 'T'))
                               AND (PROD.CDFILIAL     = :P_CDFILIAL)
                               AND (PR.CDFILIAL       = PROD.CDFILIAL)
                               AND (PROD.CDPRODUTO =   DECODE( NVL( PR.IDUTLOCESTPRACA, 'N'), 'N' , :P_CDPRODINSUMO, :P_CDPRODUTO))
                               AND (((TRUNC(LO.DTVALILOTE) >= TRUNC(SYSDATE)) AND (PROD.IDUTCTRLLOTE = 'S'))
                                OR (PR.IDLOTEVENC = 'S')
                                OR (PROD.IDUTCTRLLOTE <> 'S'))
                    
                          ) E
                    ORDER BY ORDEM, IDITEMPEDTERC DESC,
                       (CASE WHEN :P_IDTPRETIRADA ='FI' THEN E.DTENTRLOTE
                             WHEN :P_IDTPRETIRADA = 'FE' THEN E.DTVALILOTE END),
                        DECODE(:P_IDTPRETIRADA, 'LI', E.DTENTRLOTE) DESC, E.DTENTRLOTE, DTFABRPROD, NRLOTEESTQ)
                        WHERE NRLOTEESTQ = :P_NRLOTEESTQ
                        AND NRSUBLOTE = :P_NRSUBLOTE";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_CDPRODINSUMO'    => $cdproduto,
                'P_CDPRODUTO'       => $row['CDPRODUTO'],
                'P_IDPRODPILOTO'    => $row['IDPRODPILOTO'],
                'P_NRORDPROD'       => $row['NRORDPROD'],
                'P_NRSUBORDPROD'    => $row['NRSUBORDPROD'],
                'P_CDPROCFABRIC'    => $row['CDPROCFABRIC'],
                'P_CDPLANPRODLT'    => '',
                'P_IDTPRETIRADA'    => $idtpretirada,
                'P_QTRETIPROD'      => $qtretiprod,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqTercProdProducao($row, $cdproduto){
        try {
            $sql = "SELECT DISTINCT P.NMPRODUTO, P.SGUNIDADE,P.CDPRODUTO,
                           CDARVPROD
                    FROM INSUMETAPA I, PRODUTO P, ORDPRODUCAO O, ITEMPEDTERC T
                    WHERE (I.CDFILIAL     = :P_CDFILIAL    )
                      AND (I.CDPRODUTO    = :P_CDPRODUTO   )
                      AND (P.CDPRODUTO    = I.CDPRODINSUMO )
                      AND (O.CDFILIAL     = I.CDFILIAL     )
                      AND (O.CDPRODUTO    = I.CDPRODUTO    )
                      AND (O.CDPROCFABRIC = I.CDPROCFABRIC )
                      AND (O.CDPROCFABRIC = :P_CDPROCFABRIC)
                      AND (O.NRORDPROD    = :P_NRORDPROD   )
                      AND (O.NRSUBORDPROD = :P_NRSUBORDPROD)
                      AND (O.CDFILIAL     = T.CDFILIAL     )
                      AND (O.NRPEDIDO     = T.NRPEDIDO     )
                      AND (O.NRITEMPED    = T.NRITEMPED    )
                      AND (I.CDPRODINSUMO = T.CDPRODUTO    )
                      AND ((I.CDPRODINSUMO = :P_CDPRODINSUMO) OR (:P_CDPRODINSUMO = 'T'))
                    ORDER BY
                       P.CDARVPROD";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_CDPRODUTO'       => $row['CDPRODUTO'],
                'P_CDPROCFABRIC'    => $row['CDPROCFABRIC'],
                'P_NRORDPROD'       => $row['NRORDPROD'],
                'P_NRSUBORDPROD'    => $row['NRSUBORDPROD'],
                'P_CDPRODINSUMO'    => $cdproduto
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqPosTercProd($row, $cdproduto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote){
        try {
            $sql = "SELECT PO.CDFILIAL      ,
                           PO.CDPRODUTO     ,
                           PO.CDALMOXARIFE  ,
                           PO.CDLOCALESTOQ  ,
                           PO.NRLOTEESTQ    ,
                           PO.NRSUBLOTE     ,
                           PO.QTTOTALESTOQ  ,
                           LO.VRAPROVPROD   ,
                           0 AS VRCUSTMEDATU,
                           LO.DTVALILOTE
                      FROM AGENTETERCECLI AC,
                           ORDPRODUCAO    OP,
                           ITEMPEDTERC    IT,
                           POSESTATU      PO,
                           PEDIDO         PE,
                           LOTE           LO
                     WHERE (PO.CDFILIAL       = :P_CDFILIAL      )
                       AND (PO.CDPRODUTO      = :P_CDPRODINSUMO  )
                       AND (OP.NRORDPROD      = :P_NRORDPROD     )
                       AND (OP.NRSUBORDPROD   = :P_NRSUBORDPROD  )
                       AND (OP.CDPRODUTO      = :P_CDPRODUTO     )
                       AND (OP.CDPROCFABRIC   = :P_CDPROCFABRIC  )
                       AND (PO.CDALMOXARIFE   = :P_CDALMOXARIFE  )
                       AND (PO.CDLOCALESTOQ   = :P_CDLOCALESTOQ  )
                       AND (PO.NRLOTEESTQ     = :P_NRLOTEESTQ    )
                       AND (PO.NRSUBLOTE      = :P_NRSUBLOTE     )
                       AND (PO.CDFILIAL       = AC.CDFILIAL      )
                       AND (PO.CDALMOXARIFE   = AC.CDALMOXTERCCLI)
                       AND (PO.CDLOCALESTOQ   = AC.CDLOCALTERCCLI)
                       AND (PO.CDFILIAL       = LO.CDFILIAL      )
                       AND (PO.NRLOTEESTQ     = LO.NRLOTEESTQ    )
                       AND (PO.NRSUBLOTE      = LO.NRSUBLOTE     )
                       AND (PO.CDPRODUTO      = LO.CDPRODUTO     )
                       AND (IT.CDFILIAL       = OP.CDFILIAL      )
                       AND (IT.NRPEDIDO       = OP.NRPEDIDO      )
                       AND (IT.NRITEMPED      = OP.NRITEMPED     )
                       AND (IT.CDPRODUTO      = PO.CDPRODUTO     )
                       AND (PE.CDFILIAL       = IT.CDFILIAL      )
                       AND (PE.NRPEDIDO       = IT.NRPEDIDO      )
                       AND (PE.CDFILIAL       = AC.CDFILIAL      )
                       AND (PE.CDCLIENTE      = AC.CDCLIENTE     )
                       AND (PE.IDPEDINDUSTRIA = 'S'              )
                       AND (PO.QTTOTALESTOQ   > 0                )
                     ORDER BY LO.DTVALILOTE DESC";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_CDPRODUTO'       => $row['CDPRODUTO'],
                'P_CDPROCFABRIC'    => $row['CDPROCFABRIC'],
                'P_NRORDPROD'       => $row['NRORDPROD'],
                'P_NRSUBORDPROD'    => $row['NRSUBORDPROD'],
                'P_CDPRODINSUMO'    => $cdproduto,
                'P_CDALMOXARIFE'    => $cdalmoxarife,
                'P_CDLOCALESTOQ'    => $cdlocalestoq,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdFili($cdfilial, $cdproduto){
        try {
            $sql = "SELECT PF.CDALMOXPRODPL, 
                            NVL(PF.CDLOCPRODPL, ' ') CDLOCPRODPL, 
                            PF.CDALMOXPROD, 
                            NVL(PF.CDLOCPROD, ' ') CDLOCPROD,
                            PF.CDALMOXINSPR ALMOXORIGEM, 
                            PF.CDLOCALINSPR LOCALORDIGEM,
                            PF.CDALMOXINSPRPIL ALMOXLPILOTO, 
                            PF.CDLOCALINSPRPIL LOCALPILOTO, 
                            P.VRFATOCONV,
                            NVL(IDUTVRFTCONVVAR, 'N') IDUTVRFTCONVVAR,
                            IDCNTRESTOQ,
                            IDUTCTRLLOTE
                      FROM PRODFILI PF, PRODUTO P
                     WHERE PF.CDFILIAL  = :P_CDFILIAL
                       AND PF.CDPRODUTO = :P_CDPRODUTO
                       AND PF.CDPRODUTO = P.CDPRODUTO";
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    /*public function adqInsumetapa($row, $cdetapaprod, $cdproduto){
        try {
            $sql = "SELECT
                       IDLOTEPRETAP,
                       IDUTAPRREQET,
                       NVL( IDSEMIACETAP, 'N') AS IDSEMIACETAP,
                       IDCOMPATI,
                       NVL(VRFATOPADR, 1) VRFATOPADR
                    FROM
                       INSUMETAPA
                    WHERE
                       CDFILIAL     = :P_CDFILIAL     AND
                       CDPRODUTO    = :P_CDPRODUTO    AND
                       CDPROCFABRIC = :P_CDPROCFABRIC AND
                       CDETAPAPROD  = :P_CDETAPAPROD  AND
                       CDPRODINSUMO = :P_CDPRODINSUMO";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_CDPRODUTO'       => $row['CDPRODUTO'],
                'P_CDPROCFABRIC'    => $row['CDPROCFABRIC'],
                'P_CDETAPAPROD'     => $cdetapaprod,
                'P_CDPRODINSUMO'    => $cdproduto
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }*/
    
    public function acmIns_LotItRetEst($row, 
                                        $nrretest, 
                                        $nrseqretprod, 
                                        $cdproduto,
                                        
                                        $cdalmoxarife, 
                                        $cdlocalestoq, 
                                        $nrloteestq, 
                                        $nrsublote,
                                        
                                        $vraprovprodret, 
                                        
                                        $nrseqretlote, 
                                        $qtretprod, 
                                        $qtretliqprod, 
                                        $qtrejretprod, 
                                        $dtlancretdev){
        try {
            $sql = "INSERT INTO LOTITRETIEST 
                    (CDFILIAL, 
                    NRRETEST, 
                    NRSEQRETPROD, 
                    CDPRODUTO, 
                    CDALMOXARIFE, 
                    CDLOCALESTOQ,
                    NRLOTEESTQ, 
                    NRSUBLOTE, 
                    NRSEQRETLOTE, 
                    QTRETPROD, 
                    QTRETLIQPROD,
                    QTREJRETPROD, 
                    DTLANCRETDEV, 
                    VRAPROVPRODRET)
                VALUES
                    (:CDFILIAL, 
                    :NRRETEST, 
                    :NRSEQRETPROD, 
                    :CDPRODUTO, 
                    :CDALMOXARIFE, 
                    :CDLOCALESTOQ,
                    :NRLOTEESTQ, 
                    :NRSUBLOTE, 
                    :NRSEQRETLOTE, 
                    :QTRETPROD, 
                    :QTRETLIQPROD,
                    :QTREJRETPROD, 
                    :DTLANCRETDEV, 
                    :VRAPROVPRODRET)";
            $params = array(
                'CDFILIAL'        => $row['CDFILIAL'],
                'NRRETEST'        => $nrretest,
                'NRSEQRETPROD'    => $nrseqretprod,
                'CDPRODUTO'       => $cdproduto,
                'CDALMOXARIFE'    => $cdalmoxarife,
                'CDLOCALESTOQ'    => $cdlocalestoq,
                'NRLOTEESTQ'      => $nrloteestq,
                'NRSUBLOTE'       => $nrsublote,
                'NRSEQRETLOTE'    => $nrseqretlote,
                'QTRETPROD'       => $qtretprod,
                'QTRETLIQPROD'    => $qtretliqprod,
                'QTREJRETPROD'    => $qtrejretprod,
                'DTLANCRETDEV'    => $dtlancretdev,
                'VRAPROVPRODRET'  => $vraprovprodret
            );
            
            $std = $this->connection->prepare($sql);
            return $std->execute($params);
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getParametrizacaoRetiradaSetor($cdfilial){
        try {
            $sql = "SELECT
                       P.CDSETORPROD,
                       S.NMSETOR,
                       P.CDTIPORETI,
                       T.NMTIPORETI,
                       P.NRRETMAXLOT,
                       P.IDALTERAPCPRET,
                       NVL(P.NRCASASRET, 6) NRCASASRET
                    FROM
                       PARAMAN P,
                       SETOR S,
                       TIPORETI T
                    WHERE
                       P.CDFILIAL    = :P_CDFILIAL and
                       P.CDSETORPROD = S.CDSETOR(+)   and
                       P.CDTIPORETI  = T.CDTIPORETI(+)";
            $params = array(
                'P_CDFILIAL'        => $cdfilial
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getLoteUnico($cdfilial, $cdproduto, $cdprocfabric, $cdetapaprod, $cdprodinsumo){
        try {
            $sql = "SELECT NVL(IDLOTEUNICO, 'N') AS IDLOTEUNICO
                      FROM INSUMETAPA
                     WHERE CDFILIAL     = :P_CDFILIAL AND
                           CDPRODUTO    = :P_CDPRODUTO AND
                           CDPROCFABRIC = :P_CDPROCFABRIC AND
                           CDETAPAPROD  = :P_CDETAPAPROD AND
                           CDPRODINSUMO = :P_CDPRODINSUMO";
            
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_CDPRODUTO'        => $cdproduto,
                'P_CDPROCFABRIC'     => $cdprocfabric,
                'P_CDETAPAPROD'      => $cdetapaprod,
                'P_CDPRODINSUMO'     => $cdprodinsumo
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0]['IDLOTEUNICO'] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getCountProdRet($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT COUNT(1) QTPRODRET
                    FROM RETIRADAESTO
                    WHERE 
                        CDFILIAL     = :P_CDFILIAL  AND
                        NRORDPROD    = :P_NRORDPROD AND
                        NRSUBORDPROD = :P_NRSUBORDPROD";
                    
                  
            
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_NRORDPROD'        => $nrordprod,
                'P_NRSUBORDPROD'     => $nrsubordprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getVdEvolEstq($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT
                     COUNT(I.IDEVOLESTOQ) QTRETEST
                  FROM
                     RETIRADAESTO R,
                     ITLANCTOEST  I
                  WHERE
                     R.CDFILIAL     = :P_CDFILIAL      AND
                     R.NRORDPROD    = :P_NRORDPROD     AND
                     R.NRSUBORDPROD = :P_NRSUBORDPROD  AND
                     I.CDFILIAL     = R.CDFILIAL       AND
                     I.NRLANCESTQ   = R.NRLANCESTQ     AND
                     I.IDEVOLESTOQ  = 'S'";
                    
                   
            
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_NRORDPROD'        => $nrordprod,
                'P_NRSUBORDPROD'     => $nrsubordprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }  
    }
    
    public function adqInsuPrev($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT ITEMRETIEST.CDFILIAL ,
                      ITEMRETIEST.NRRETEST ,
                      ITEMRETIEST.NRSEQRETPROD ,
                      ITEMRETIEST.CDPRODUTO ,
                      ITEMRETIEST.QTPRERETPROD ,
                      ITEMRETIEST.DTRETPROD ,
                      ITEMRETIEST.CDTIPORETI ,
                      ITEMRETIEST.VRPROPORINSU ,
                      ITEMRETIEST.CDTIPOINSU ,
                      RETIRADAESTO.NRORDPROD ,
                      RETIRADAESTO.NRSUBORDPROD ,
                      RETIRADAESTO.CDETAPAPROD ,
                      RETIRADAESTO.CDCENTCUST ,
                      RETIRADAESTO.CDSETOR ,
                      RETIRADAESTO.NRLANCESTQ ,
                      ETAPAPROD.NMETAPAPROD ,
                      PRODUTO.CDARVPROD ,
                      NVL(ITEMRETIEST.NMINSUMO, PRODUTO.NMPRODUTO) NMPRODUTO,
                      PRODUTO.SGUNIDADE ,
                      PRODUTO.IDPRODPROD ,
                      TIPORETI.NMTIPORETI ,
                      CENTCUST.NMCENTCUST ,
                      'N' AS IDCHECK ,
                      ''  AS IDCALC ,
                      SETOR.NMSETOR ,
                      RETIRADAESTO.IDPESPRODRET ,
                      DECODE(ITEMRETIEST.CDPRODUTO, PARAMAN.CDPRODAGUA, 'S', MULTPRODUTOS.CDPRODUTO, 'S', 'N') AS IDPRODAGUA,
                      NVL(RETIRADAESTO.IDAGUARETIRADA, 'N')                                                    AS IDAGUARETIRADA,
                      NVL(TIPORETI.IDTIPOMOVI, 'A')                                                            AS IDTIPOMOVI,
                      PRODUTO.IDTEOROBRIG,
                      ITEMRETIEST.DSFASE
                    FROM TIPORETI,
                      ETAPAPROD,
                      CENTCUST,
                      SETOR,
                      PRODUTO,
                      RETIRADAESTO,
                      ITEMRETIEST ,
                      PARAMAN,
                      MULTPRODUTOS,
                      ( SELECT DISTINCT COMPPROD.CDPRODUTO,
                        COMPPROD.CDPRODINSUMO,
                        COMPPROD.CDESPEPROD,
                        COMPPROD.NRORDINSUMO
                      FROM COMPPROD,
                        TCOMPPRO
                      WHERE COMPPROD.CDPRODUTO =
                        (SELECT OP.CDPRODUTO
                        FROM ORDPRODUCAO OP
                        WHERE OP.CDFILIAL   = :P_CDFILIAL
                        AND OP.NRORDPROD    = :P_NRORDPROD
                        AND OP.NRSUBORDPROD = :P_NRSUBORDPROD
                        )
                      AND COMPPROD.CDESPEPROD =
                        (SELECT E.CDESPEPROD
                        FROM ESPEPROD E
                        WHERE E.CDPRODUTO  = COMPPROD.CDPRODUTO
                        AND E.DTVALESPEPRO =
                          (SELECT MAX(X.DTVALESPEPRO) FROM ESPEPROD X WHERE X.CDPRODUTO = E.CDPRODUTO
                          )
                        )
                      AND COMPPROD.CDTIPOCOMP  = TCOMPPRO.CDTIPOCOMP
                      AND COMPPROD.CDPRODUTO   = TCOMPPRO.CDPRODUTO
                      AND COMPPROD.CDESPEPROD  = TCOMPPRO.CDESPEPROD
                      AND TCOMPPRO.IDTCOMPPROC = 'S'
                      ) C
                    WHERE RETIRADAESTO.CDFILIAL   = :P_CDFILIAL
                    AND RETIRADAESTO.NRORDPROD    = :P_NRORDPROD
                    AND RETIRADAESTO.NRSUBORDPROD = :P_NRSUBORDPROD
                    AND RETIRADAESTO.CDFILIAL     = ITEMRETIEST.CDFILIAL
                    AND RETIRADAESTO.NRRETEST     = ITEMRETIEST.NRRETEST
                    AND RETIRADAESTO.CDCENTCUST   = CENTCUST.CDCENTCUST(+)
                    AND RETIRADAESTO.CDSETOR      = SETOR.CDSETOR (+)
                    AND RETIRADAESTO.CDETAPAPROD  = ETAPAPROD.CDETAPAPROD
                    AND ITEMRETIEST.CDPRODUTO     = PRODUTO.CDPRODUTO
                    AND ITEMRETIEST.CDTIPORETI    = TIPORETI.CDTIPORETI(+)
                    AND ITEMRETIEST.CDPRODUTO     = C.CDPRODINSUMO (+)
                    AND RETIRADAESTO.CDFILIAL     = PARAMAN.CDFILIAL
                    AND ITEMRETIEST.CDFILIAL      = MULTPRODUTOS.CDFILIAL(+)
                    AND ITEMRETIEST.CDPRODUTO     = MULTPRODUTOS.CDPRODUTO(+)
                    ORDER BY RETIRADAESTO.CDETAPAPROD ,
                      NVL(LPAD(C.NRORDINSUMO,2,0),999),
                      ITEMRETIEST.CDPRODUTO";

                   
            
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'        => $nrordprod,
                'P_NRSUBORDPROD'        => $nrsubordprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }  
    }
    
    
    public function adqVdQtPrevDifMov($cdfilial, $nrretest, $cdproduto, $nrseqretprod){
        try {
            $sql = "SELECT I.CDFILIAL, I.NRRETEST, I.QTPRERETPROD , L.QTMOVIMENTADA
                    FROM ITEMRETIEST I, PRODFILI PF, (SELECT NVL(SUM(QTRETPROD),0) AS QTMOVIMENTADA
                                         FROM  LOTITRETIEST
                                        WHERE CDFILIAL = :P_CDFILIAL
                                          AND NRRETEST = :P_NRRETEST
                                          AND CDPRODUTO = :P_CDPRODUTO
                                          AND NRSEQRETPROD = :P_NRSEQRETPROD) L
                  WHERE I.CDFILIAL = :P_CDFILIAL
                    AND I.NRRETEST = :P_NRRETEST
                    AND I.CDFILIAL = PF.CDFILIAL
                    AND I.CDPRODUTO = PF.CDPRODUTO
                    AND NVL(PF.IDCNTRESTOQ, 'S') = 'S'
                    AND ROUND(I.QTPRERETPROD, 5) <> ROUND(NVL(L.QTMOVIMENTADA,0),5))";
                           
                    
            
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_NRRETEST'        => $nrretest,
                'P_CDPRODUTO'     => $cdproduto,
                'P_NRSEQRETPROD'      => $nrseqretprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqParaman($cdfilial){
        try {
            $sql = "SELECT IDBLOQFECHA,
                      IDVALANARETPROD,
                      IDVALINSPRETPRO,
                      IDUTAUTSENHARET,
                      IDALTERARETI,
                      IDVDSEMIVDACAB,
                      IDRETPERDAUTO,
                      IDBLOQRETMPREV,
                      IDVDINSUVDACA,
                      IDRECALCINSUMO,
                      IDCODBARRA,
                      NVL(IDUTBALCWTFECHOP, 'N') IDUTBALCWTFECHOP,
                      NVL(IDINFPESOMANUAL, 'N') IDINFPESOMANUAL,
                      IDTIPOCONEXAO,
                      IDMODELOBALANCA,
                      CDPORTABALANCA,
                      IDTPRETIRADA,
                      IDDTLOTELANCOP,
                      IDVALIDADEPROD,
                      IDRECALCQTOP,
                      IDUTEQUIPLAPROD
                    FROM PARAMAN
                    WHERE CDFILIAL = :P_CDFILIAL";
            
            $params = array(
                'P_CDFILIAL'         => $cdfilial
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqParamGeral(){
        try {
            $sql = "SELECT 
                     IDUTILDTPROC   
                  FROM 
                     PARAMGERAL";

            $params = array();
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqParamfilial($cdfilial){
        try {
            $sql = "SELECT IDESTONLINE, IDPERMESTNEG, IDUTILLOTE, IDUTILLCESTQ,
                            IDUTILALMOX, TO_CHAR(DTPROCESSA, 'DD/MM/YYYY') DTPROCESSA, IDLANCDTMENFIL, NRDIASSAIDFIL,
                            IDLANCDTMAIFIL, NRDIASSAIDPDFIL, IDCTRLESTQ, NRDIASENTRPDFIL, NRDIASENTRFIL
                        FROM PARAMFILIAL
                        WHERE CDFILIAL = :P_CDFILIAL";
            $params = array(
                'P_CDFILIAL'         => $cdfilial
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updateRetiradaAgua($cdfilial, $nrretest){
        try {
            $sql = "UPDATE RETIRADAESTO
                     SET IDAGUARETIRADA = 'S'
                    WHERE CDFILIAL      = :P_CDFILIAL AND
                          NRRETEST      = :P_NRRETEST";
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_NRRETEST'         => $nrretest
            );
            
            $std = $this->connection->prepare($sql);
            return $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqParamMaf($cdfilial){
        try {
            $sql = "SELECT
                     P.IDVRUNITPRACAB,
                     P.CDSETORPROD,
                     S.NMSETOR,
                     P.CDTIPORETI,
                     T.NMTIPORETI,
                     P.NRRETMAXLOT,
                     P.IDALTERAPCPRET,
                     NVL(P.NRCASASRET, 6) NRCASASRET
                  FROM
                     PARAMAN P,
                     SETOR S,
                     TIPORETI T
                  WHERE
                     P.CDFILIAL    = :P_CDFILIAL AND
                     P.CDSETORPROD = S.CDSETOR(+)   AND
                     P.CDTIPORETI  = T.CDTIPORETI(+)";
            $params = array(
                'P_CDFILIAL'         => $cdfilial
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqProcFabrica($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT O.CDPROCFABRIC, O.CDPRODUTO, P.IDVERIFCOMPAT
                        FROM ORDPRODUCAO O, PROCFABRICA P
                       WHERE O.CDFILIAL = :P_CDFILIAL AND
                             O.NRORDPROD    = :P_NRORDPROD AND
                             O.NRSUBORDPROD = :P_NRSUBORDPROD AND
                             O.CDFILIAL = P.CDFILIAL AND
                             O.CDPROCFABRIC = P.CDPROCFABRIC AND
                             O.CDPRODUTO = P.CDPRODUTO";
                     
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_NRORDPROD'        => $nrordprod,
                'P_NRSUBORDPROD'     => $nrsubordprod
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqLoteUnico($cdfilial, $cdproduto, $cdprocfabric, $cdetapaprod, $cdprodinsumo){
        try {
            $sql = "SELECT NVL(IDLOTEUNICO, 'N') AS IDLOTEUNICO
                        FROM INSUMETAPA
                       WHERE CDFILIAL     = :P_CDFILIAL AND
                             CDPRODUTO    = :P_CDPRODUTO AND
                             CDPROCFABRIC = :P_CDPROCFABRIC AND
                             CDETAPAPROD  = :P_CDETAPAPROD AND
                             CDPRODINSUMO = :P_CDPRODINSUMO";
                             
                             
                     
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_CDPRODUTO'        => $cdproduto,
                'P_CDPROCFABRIC'     => $cdprocfabric,
                'P_CDETAPAPROD'      => $cdetapaprod,
                'P_CDPRODINSUMO'     => $cdprodinsumo
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqVDQtdeLote($cdfilial, $cdproduto,  $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT DISTINCT LT.NRLOTEESTQ
                        FROM LOTITRETIEST LT, RETIRADAESTO RE
                       WHERE LT.CDFILIAL = :P_CDFILIAL
                         AND LT.CDPRODUTO = :P_CDPRODUTO
                         AND LT.NRRETEST = RE.NRRETEST
                         AND LT.CDFILIAL = RE.CDFILIAL
                         AND RE.NRORDPROD = :P_NRORDPROD
                         AND RE.NRSUBORDPROD = :P_NRSUBORDPROD";
                             
                             
                     
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_CDPRODUTO'        => $cdproduto,
                'P_NRORDPROD'        => $nrordprod,
                'P_NRSUBORDPROD'     => $nrsubordprod
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function xqryNomeProd($cdproduto){
        try {
            $sql = "SELECT NMPRODUTO FROM PRODUTO WHERE CDPRODUTO = :P_CDPRODUTO";
                             
                             
                     
            $params = array(
                'P_CDPRODUTO'        => $cdproduto
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqInsumoComp($cdfilial, $cdproduto, $cdprocfabric){
        try {
            $sql = "SELECT I.CDFILIAL, I.CDPRODINSUMO AS CDPRODUTO, P.NMPRODUTO
                     FROM INSUMETAPA I, PRODUTO P
                    WHERE I.CDFILIAL     = :P_CDFILIAL     AND
                          I.CDPRODUTO    = :P_CDPRODUTO    AND
                          I.CDPROCFABRIC = :P_CDPROCFABRIC AND
                          I.IDCOMPATI    = 'S'             AND
                          I.CDPRODINSUMO = P.CDPRODUTO";
                             
            
       
                     
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto,
                'P_CDPROCFABRIC'    => $cdprocfabric
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqLotesRetComp($cdfilial, $nrordprod, $nrsubordprod, $cdproduto){
        try {
            $sql = " SELECT DISTINCT
                         SUBSTR(P.NMPRODUTO, 1, 18) AS NMPRODUTO,
                         R.CDALMOXARIFE,
                         R.CDLOCALESTOQ,
                         R.NRLOTEESTQ,
                         R.NRSUBLOTE,
                         R.CDPRODUTO,
                         R.IDTPRETEST
                      FROM
                         (SELECT
                             L.CDPRODUTO,
                             L.CDALMOXARIFE,
                             L.CDLOCALESTOQ,
                             L.NRLOTEESTQ,
                             L.NRSUBLOTE,
                             L.IDTPRETEST
                          FROM
                             RETIRADAESTO R,
                             ITEMRETIEST I,
                             LOTITRETIEST L, INSUMETAPA E
                          WHERE
                             (R.CDFILIAL     = :P_CDFILIAL)        AND
                             (R.NRORDPROD    = :P_NRORDPROD)       AND
                             (R.NRSUBORDPROD = :P_NRSUBORDPROD)    AND
                             (I.CDFILIAL     = R.CDFILIAL)         AND
                             (I.NRRETEST     = R.NRRETEST)         AND
                             (L.CDFILIAL     = I.CDFILIAL)         AND
                             (L.NRRETEST     = I.NRRETEST)         AND
                             (L.NRSEQRETPROD = I.NRSEQRETPROD)     AND
                             (L.CDPRODUTO    = I.CDPRODUTO)        AND
                             (E.CDFILIAL     = R.CDFILIAL)         AND
                             (E.CDPRODINSUMO = I.CDPRODUTO)        AND
                             (E.CDPRODUTO    = :P_CDPRODUTO)       AND
                             (E.IDCOMPATI = 'S')                   AND
                             (L.IDTPRETEST = 'R')
                         ) R,
                         PRODUTO P
                      WHERE
                         R.CDPRODUTO = P.CDPRODUTO";
                             
            
           
       
                     
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod,
                'P_CDPRODUTO'       => $cdproduto
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqAnalisesComp($cdfilial, $cdprocfabric, $cdproduto, $cdprodinsumo, $nrloteestq){
        try {
            $sql = "SELECT DISTINCT
                         A.CDFILIAL, A.NRAUDITORIA
                    FROM AUDITORIA A, INSUMETAPA I, FORMULAUDITANA F
                   WHERE A.CDFILIAL          = :P_CDFILIAL     AND
                         A.CDFILIAL          = I.CDFILIAL      AND
                         A.CDFILIAL          = F.CDFILIAL      AND
                         A.CDPRODUTO         = I.CDPRODUTO     AND
                         A.IDCOMPATIBILIDADE = 'S'             AND
                         A.NRAUDITORIA       = F.NRAUDITORIA   AND
                         I.CDPROCFABRIC      = :P_CDPROCFABRIC AND
                         I.CDPRODUTO         = :P_CDPRODUTO    AND
                         (INSTR(:P_CDPRODINSUMO, I.CDPRODINSUMO) <> 0) AND
                         (INSTR(:P_NRLOTEESTQ, F.NRLOTEESTQ) <> 0) AND
                         --I.CDETAPAPROD       = :P_CDETAPAPROD  AND
                         I.IDCOMPATI         = 'S'             AND
                         I.CDPRODINSUMO      = F.CDPRODUTO     AND
                         F.IDSITAMOSTRA      = '2'             AND
                         A.IDSTATUSAUDI      = '4'             AND
                         A.DTVALANA          > SYSDATE
                   ORDER BY NRAUDITORIA DESC";
                             
                     
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPROCFABRIC'       => $cdprocfabric,
                'P_CDPRODUTO'    => $cdproduto,
                'P_CDPRODINSUMO'    => $cdprodinsumo,
                'P_NRLOTEESTQ'    => $nrloteestq
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqLotesRetDev($cdfilial, $cdproduto, $nrretest){
        try {
            $sql = "SELECT DISTINCT
                     SUBSTR(P.NMPRODUTO, 1, 18) AS NMPRODUTO,
                     R.CDALMOXARIFE,
                     R.CDLOCALESTOQ,
                     R.NRLOTEESTQ,
                     R.NRSUBLOTE,
                     R.IDTPRETEST
                  FROM
                     (SELECT
                         L.CDPRODUTO,
                         L.CDALMOXARIFE,
                         L.CDLOCALESTOQ,
                         L.NRLOTEESTQ,
                         L.NRSUBLOTE,
                         L.IDTPRETEST
                      FROM
                         RETIRADAESTO R,
                         ITEMRETIEST I,
                         LOTITRETIEST L
                      WHERE
                         (R.CDFILIAL     = :P_CDFILIAL)        AND
                         (INSTR(:P_NRRETEST, R.NRRETEST) <> 0) AND
                         (I.CDFILIAL     = R.CDFILIAL)         AND
                         (I.NRRETEST     = R.NRRETEST)         AND
                         (I.CDPRODUTO    = :P_CDPRODUTO)    AND
                         (L.CDFILIAL     = I.CDFILIAL)         AND
                         (L.NRRETEST     = I.NRRETEST)         AND
                         (L.NRSEQRETPROD = I.NRSEQRETPROD)     AND
                         (L.CDPRODUTO    = I.CDPRODUTO)
                     ) R,
                     PRODUTO P
                  WHERE
                     R.CDPRODUTO = P.CDPRODUTO";
                    
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRRETEST'        => $nrretest
            );
            
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqVdMov($cdfilial, $cdproduto, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT NVL(R.QTDE_RETIRADA, 0) - NVL( D.QTDE_DEVOLVIDA, 0) AS QTSALDO, P.NMPRODUTO
                    FROM PRODUTO P,
                    ( SELECT
                         SUM(L.QTRETPROD) AS QTDE_RETIRADA
                      FROM
                         RETIRADAESTO R,
                         ITEMRETIEST I,
                         LOTITRETIEST L
                      WHERE
                         (R.CDFILIAL     = :P_CDFILIAL)     AND
                         (R.NRORDPROD    = :P_NRORDPROD)    AND
                         (R.NRSUBORDPROD = :P_NRSUBORDPROD) AND
                         (I.CDFILIAL     = R.CDFILIAL)      AND
                         (I.NRRETEST     = R.NRRETEST)      AND
                         (I.CDPRODUTO    = :P_CDPRODUTO)    AND
                         (L.CDFILIAL     = I.CDFILIAL)      AND
                         (L.NRRETEST     = I.NRRETEST)      AND
                         (L.NRSEQRETPROD = I.NRSEQRETPROD)  AND
                         (L.CDPRODUTO    = I.CDPRODUTO)     AND
                         (IDTPRETEST      = 'R'  )          AND
                         (R.NRLANCESTQ IS NOT NULL)) R,
                  
                     (SELECT
                         SUM(L.QTRETPROD) AS QTDE_DEVOLVIDA
                      FROM
                         RETIRADAESTO R,
                         ITEMRETIEST I,
                         LOTITRETIEST L
                      WHERE
                         (R.CDFILIAL     = :P_CDFILIAL)     AND
                         (R.NRORDPROD    = :P_NRORDPROD)    AND
                         (R.NRSUBORDPROD = :P_NRSUBORDPROD) AND
                         (I.CDFILIAL     = R.CDFILIAL)      AND
                         (I.NRRETEST     = R.NRRETEST)      AND
                         (I.CDPRODUTO    = :P_CDPRODUTO)    AND
                         (L.CDFILIAL     = I.CDFILIAL)      AND
                         (L.NRRETEST     = I.NRRETEST)      AND
                         (L.NRSEQRETPROD = I.NRSEQRETPROD)  AND
                         (L.CDPRODUTO    = I.CDPRODUTO) AND
                         (IDTPRETEST      = 'D'  ))  D
                  WHERE
                     P.CDPRODUTO = :P_CDPRODUTO";
                             
       
                     
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function qryInfoProd($cdfilial, $cdproduto, $nrloteestq, $nrsublote){
        try {
            $sql = "SELECT NRLOTEESTQ, NMPRODUTO, IDSTANALISE, DTVALANALOTE, DTENTRLOTE,
                      TO_CHAR(DTVALILOTE, 'MM/DD/YYYY') DTVALILOTE, IDSTINSPECAO
                      FROM LOTE L, PRODUTO P 
                      WHERE    
                         L.CDFILIAL   = :P_CDFILIAL AND
                         L.CDPRODUTO  = :P_CDPRODUTO AND
                         L.NRLOTEESTQ = :P_NRLOTEESTQ AND 
                         L.NRSUBLOTE  = :P_NRSUBLOTE  AND
                         P.CDPRODUTO  = L.CDPRODUTO ";
                         
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqMaxAudit($cdfilial, $cdproduto, $nrloteestq, $nrsublote){
        try {
            $sql = "SELECT MAX(NRAUDITORIA) AS NRAUDITORIA, DTVALANA, IDLIBERAUDIT
                       FROM AUDITORIA
                        WHERE CDFILIAL     = :P_CDFILIAL   AND
                              CDPRODUTO    = :P_CDPRODUTO  AND
                              NRLOTEESTQ   = :P_NRLOTEESTQ AND
                              NRSUBLOTE    = :P_NRSUBLOTE  AND
                              IDORIGAUDIT  = 'ANA'        AND
                              IDSTATUSAUDI = '4'
                       GROUP BY DTVALANA, IDLIBERAUDIT
                      ORDER BY 1 DESC";

            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqReanalisePen($cdfilial, $cdproduto, $nrloteestq, $nrsublote){
        try {
            $sql = "SELECT NRAUDITORIA
                       FROM AUDITORIA
                       WHERE CDFILIAL     = :P_CDFILIAL   
                         AND CDPRODUTO    = :P_CDPRODUTO  
                         AND NRLOTEESTQ   = :P_NRLOTEESTQ 
                         AND NRSUBLOTE    = :P_NRSUBLOTE  
                         AND IDORIGAUDIT  = 'ANA'
                         AND IDSTATUSAUDI <> '4'
                         AND DTINCLAUDIT = (SELECT MAX(DTINCLAUDIT)
                                             FROM AUDITORIA
                                            WHERE CDFILIAL     = :P_CDFILIAL   
                                              AND CDPRODUTO    = :P_CDPRODUTO  
                                              AND NRLOTEESTQ   = :P_NRLOTEESTQ 
                                              AND NRSUBLOTE    = :P_NRSUBLOTE  
                                              AND IDORIGAUDIT  = 'ANA')";

            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function insertLanctoestoq($cdfilial, $nrlanctoest, $date, $tipolancto, $idaltcusmed, $identrsaidop, $cdfilimovi, $cdoperador, $dslancestq){
        try {
            
            $sql = "INSERT INTO LANCTOESTOQ (CDFILIAL, NRLANCESTQ, DTLANCESTQ, IDTPLANCTO, 
                                            IDALTCUSMED, IDENTRSAIDOP, CDFILIMOVI, CDOPERADOR, DSLANCESTQ)
                                    VALUES (:P_CDFILIAL, :P_NRLANCESTQ, :P_DTLANCESTQ, :P_IDTPLANCTO, 
                                            :P_IDALTCUSMED, :P_IDENTRSAIDOP, :P_CDFILIMOVI, :P_CDOPERADOR, :P_DSLANCESTQ)";
                      
                       
                      
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_NRLANCESTQ'       => $nrlanctoest,
                'P_DTLANCESTQ'       => $date,
                'P_IDTPLANCTO'       => $tipolancto,
                'P_IDALTCUSMED'      => $idaltcusmed,
                'P_IDENTRSAIDOP'     => $identrsaidop,
                'P_CDFILIMOVI'       => $cdfilimovi,
                'P_CDOPERADOR'       => $cdoperador,
                'P_DSLANCESTQ'       => $dslancestq
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updtNrlanEstqRetiesto($cdfiliallan, $nrlanctoest, $cdfilial, $nrretest){
        try {
            
            $sql = "UPDATE RETIRADAESTO SET CDFILIALLAN = :P_CDFILIALLAN, 
                                               NRLANCESTQ  = :P_NRLANCESTQ
                                            where CDFILIAL = :P_CDFILIAL AND
                                                  NRRETEST = :P_NRRETEST";
                      
            $params = array(
                'P_CDFILIALLAN'    => $cdfiliallan,
                'P_NRLANCESTQ'     => $nrlanctoest,
                'P_CDFILIAL'       => $cdfilial,
                'P_NRRETEST'       => $nrretest
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verifItemRetiEst($cdfilial, $nrretest, $cdproduto){
        try {
            $sql = "SELECT LOTITRETIEST.CDFILIAL     ,
                     LOTITRETIEST.CDFILIAL AS CDFILIMOVI,
                     LOTITRETIEST.NRRETEST     ,
                     LOTITRETIEST.NRSEQRETPROD ,
                     LOTITRETIEST.CDPRODUTO    ,
                     LOTITRETIEST.CDPRODUTO AS CDPRODMOVI,
                     LOTITRETIEST.NRLOTEESTQ   ,
                     LOTITRETIEST.NRSUBLOTE    ,
                     LOTITRETIEST.CDALMOXARIFE ,
                     LOTITRETIEST.CDLOCALESTOQ ,
                     LOTITRETIEST.QTRETPROD    ,
                     LOTITRETIEST.QTDEVRETPROD ,
                     LOTITRETIEST.QTEXTRETPROD ,
                     LOTITRETIEST.QTREJRETPROD ,
                     LOTITRETIEST.QTRETLIQPROD ,
                     LOTITRETIEST.IDTPRETEST   ,
                     LOTITRETIEST.NRSEQRETLOTE ,
                     LOTITRETIEST.DTLANCRETDEV ,
                     ITEMRETIEST.QTPRERETPROD  ,
                     ITEMRETIEST.CDTIPORETI    ,
                     RETIRADAESTO.CDCENTCUST
                  FROM
                     RETIRADAESTO, ITEMRETIEST, LOTITRETIEST
                  WHERE
                     LOTITRETIEST.CDFILIAL    = :P_CDFILIAL               AND
                     LOTITRETIEST.NRRETEST    = :P_NRRETEST               AND
                     ITEMRETIEST.CDPRODUTO    = :P_CDPRODUTO              AND
                     ITEMRETIEST.CDFILIAL     = LOTITRETIEST.CDFILIAL     AND
                     ITEMRETIEST.NRRETEST     = LOTITRETIEST.NRRETEST     AND
                     ITEMRETIEST.NRSEQRETPROD = LOTITRETIEST.NRSEQRETPROD AND
                     ITEMRETIEST.CDPRODUTO    = LOTITRETIEST.CDPRODUTO    AND
                     ITEMRETIEST.CDFILIAL     = RETIRADAESTO.CDFILIAL     AND
                     ITEMRETIEST.NRRETEST     = RETIRADAESTO.NRRETEST";
                                              
            

            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRRETEST'      => $nrretest,
                'P_CDPRODUTO'       => $cdproduto
            );
            
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updtDtLanctoEstoq($cdfilial, $stnrlanctoest, $dtlancestq){
        
        try {
            
            $sql = "UPDATE LANCTOESTOQ
                     SET DTLANCESTQ = :P_DTLANCESTQ
                   WHERE CDFILIAL   = :P_CDFILIAL
                     AND NRLANCESTQ = :P_NRLANCESTQ";
                      
            $params = array(
                'P_DTLANCESTQ'    => $dtlancestq,
                'P_CDFILIAL'      => $cdfilial,
                'P_NRLANCESTQ'    => $stnrlanctoest
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insertItLancToEst($cdfilial, $stnrlanctoest, $nrseqitem, $cdproduto, $qtlanctoest, 
                                    $qttotlancto, $cdprodmovi, $cdfilimovi, $cdalmoxarife, $cdlocalestoq, 
                                    $nrloteestq, $nrsublote, $identrsaidop, $idtipomovi, $vrlanctobrut, 
                                    $vrunilancto, $vrlanctoest, $vrcustentr, $vrtotlancto, $dslancestit, 
                                    $dtlancmovi, $idordemclc){
        try {
            
            $sql = "INSERT INTO ITLANCTOEST (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, QTLANCTOEST, 
                                            QTTOTLANCTO, CDPRODMOVI, CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ, 
                                            NRLOTEESTQ, NRSUBLOTE, IDENTRSAIDOP, IDTIPOMOVI, VRLANCTOBRUT, 
                                            VRUNILANCTO, VRTOTLANCTO, VRCUSTENTR, VRLANCTOEST, DSLANCESTIT, 
                                            DTLANCMOVI, IDORDEMCLC)
                                          VALUES (:P_CDFILIAL, :P_NRLANCESTQ, :P_NRSEQUITEM, :P_CDPRODUTO, :P_QTLANCTOEST, 
                                            :P_QTTOTLANCTO, :P_CDPRODMOVI, :P_CDFILIMOVI, :P_CDALMOXARIFE,  :P_CDLOCALESTOQ,
                                            :P_NRLOTEESTQ, :P_NRSUBLOTE, :P_IDENTRSAIDOP, :P_IDTIPOMOVI, :P_VRLANCTOBRUT, 
                                            :P_VRUNILANCTO, :P_VRTOTLANCTO, :P_VRCUSTENTR, :P_VRLANCTOEST, :P_DSLANCESTIT, 
                                            :P_DTLANCMOVI, :P_IDORDEMCLC)";
                      
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRLANCESTQ'      => $stnrlanctoest,
                'P_NRSEQUITEM'      => $nrseqitem,
                'P_CDPRODUTO'       => $cdproduto,
                'P_QTLANCTOEST'     => $qtlanctoest,
                'P_QTTOTLANCTO'     => $qttotlancto,
                'P_CDPRODMOVI'      => $cdprodmovi,
                'P_CDFILIMOVI'      => $cdfilimovi,
                'P_CDALMOXARIFE'    => $cdalmoxarife,
                'P_CDLOCALESTOQ'    => $cdlocalestoq,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote,
                'P_IDENTRSAIDOP'    => $identrsaidop,
                'P_IDTIPOMOVI'      => $idtipomovi,
                'P_VRLANCTOBRUT'    => $vrlanctobrut,
                'P_VRUNILANCTO'     => $vrunilancto,
                'P_VRTOTLANCTO'     => $vrlanctoest,
                'P_VRCUSTENTR'      => $vrcustentr,
                'P_VRLANCTOEST'     => $vrtotlancto,
                'P_DSLANCESTIT'     => $dslancestit,
                'P_DTLANCMOVI'      => $dtlancmovi,
                'P_IDORDEMCLC'      => $idordemclc
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insertItLancToEstReti($cdfilial, $stnrlanctoest, $nrseqitem, $cdproduto, $qtlanctoest, 
                                    $qttotlancto, $cdprodmovi, $cdfilimovi, $cdalmoxarife, $cdlocalestoq, 
                                    $nrloteestq, $nrsublote, $identrsaidop, $idtipomovi, $cdtiporeti,
                                    $cdcentcust, $dtlancmovi, $vrunilancto, $vrtotlancto, $vrlanctoest,
                                    $vrlanctobrut, $idordemclc){
        try {
            
            $sql = "INSERT INTO ITLANCTOEST
                        (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, QTLANCTOEST, QTTOTLANCTO,
                        CDPRODMOVI, CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE,
                        IDENTRSAIDOP, IDTIPOMOVI, CDTIPORETI, CDCENTCUST, DTLANCMOVI, VRUNILANCTO,
                        VRTOTLANCTO, VRLANCTOEST, VRLANCTOBRUT, IDORDEMCLC)
                    VALUES
                        (:P_CDFILIAL, :P_NRLANCESTQ, :P_NRSEQUITEM, :P_CDPRODUTO, :P_QTLANCTOEST, :P_QTTOTLANCTO,
                        :P_CDPRODMOVI, :P_CDFILIMOVI, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE,
                        :P_IDENTRSAIDOP, :P_IDTIPOMOVI, :P_CDTIPORETI, :P_CDCENTCUST, :P_DTLANCMOVI, :P_VRUNILANCTO,
                        :P_VRTOTLANCTO, :P_VRLANCTOEST, :P_VRLANCTOBRUT, :P_IDORDEMCLC)";
                      
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRLANCESTQ'      => $stnrlanctoest,
                'P_NRSEQUITEM'      => $nrseqitem,
                'P_CDPRODUTO'       => $cdproduto,
                'P_QTLANCTOEST'     => $qtlanctoest,
                'P_QTTOTLANCTO'     => $qttotlancto,
                'P_CDPRODMOVI'      => $cdprodmovi,
                'P_CDFILIMOVI'      => $cdfilimovi,
                'P_CDALMOXARIFE'    => $cdalmoxarife,
                'P_CDLOCALESTOQ'    => $cdlocalestoq,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote,
                'P_IDENTRSAIDOP'    => $identrsaidop,
                'P_IDTIPOMOVI'      => $idtipomovi,
                'P_CDTIPORETI'      => $cdtiporeti,
                'P_CDCENTCUST'      => $cdcentcust,
                'P_DTLANCMOVI'      => $dtlancmovi,
                'P_VRUNILANCTO'     => $vrunilancto,
                'P_VRTOTLANCTO'     => $vrtotlancto,
                'P_VRLANCTOEST'     => $vrlanctoest,
                'P_VRLANCTOBRUT'    => $vrlanctobrut,
                'P_IDORDEMCLC'      => $idordemclc
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqSeqItemLanc($cdfilial, $nrlancestq){
        try {
            $sql = "SELECT
                         NVL(MAX(NRSEQUITEM), '0')+1 AS NRSEQUITEM
                      FROM
                         ITLANCTOEST
                      WHERE
                         CDFILIAL   = :P_CDFILIAL AND
                         NRLANCESTQ = :P_NRLANCESTQ";
                    

            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRLANCESTQ'       => $nrlancestq
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0]['NRSEQUITEM'] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getCustoProdutoLcest($cdfilial, $cdproduto, $nrtipocusto, $dtreferencia, $dtatual, $almoxarife, $localestoq, $nrloteestq, $nrsublote){
        try {
            $sql = "SELECT CUSTO_PRODUTO_LCEST(:P_CDFILIAL, :P_CDPRODUTO, :P_NRTIPOCUSTO, :P_DTREFERENCIA,
                                                :P_DTATUAL, :P_ALMOXARIFE, :P_LOCALESTOQ, :P_NRLOTEESTQ,
                                                :P_NRSUBLOTE) AS CUSTO FROM DUAL";
                        

            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRTIPOCUSTO'     => $nrtipocusto,
                'P_DTREFERENCIA'    => $dtreferencia,
                'P_DTATUAL'         => $dtatual,
                'P_ALMOXARIFE'      => $almoxarife,
                'P_LOCALESTOQ'      => $localestoq,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result['CUSTO'];
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updtLotItRetiEst($nrlancestq,
                                    $nrsequitem,
                                    $cdfilial,
                                    $nrretest,
                                    $nrseqretprod,
                                    $cdproduto,
                                    $nrloteestq,
                                    $nrsublote,
                                    $cdalmoxarife,
                                    $cdlocalestoq,
                                    $nrseqretlote){
        
        try {
            
            $sql = "UPDATE LOTITRETIEST SET
                                NRLANCESTQ   = :P_NRLANCESTQ,
                                NRSEQUITEM   = :P_NRSEQUITEM
                        WHERE   CDFILIAL     = :P_CDFILIAL     AND
                                NRRETEST     = :P_NRRETEST     AND
                                NRSEQRETPROD = :P_NRSEQRETPROD AND
                                CDPRODUTO    = :P_CDPRODUTO    AND
                                NRLOTEESTQ   = :P_NRLOTEESTQ   AND
                                NRSUBLOTE    = :P_NRSUBLOTE    AND
                                CDALMOXARIFE = :P_CDALMOXARIFE AND
                                CDLOCALESTOQ = :P_CDLOCALESTOQ AND
                                NRSEQRETLOTE = :P_NRSEQRETLOTE";
                     
            $params = array(
                'P_NRLANCESTQ'      => $nrlancestq,
                'P_NRSEQUITEM'      => $nrsequitem,
                'P_CDFILIAL'        => $cdfilial,
                'P_NRRETEST'        => $nrretest,
                'P_NRSEQRETPROD'    => $nrseqretprod,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote,
                'P_CDALMOXARIFE'    => $cdalmoxarife,
                'P_CDLOCALESTOQ'    => $cdlocalestoq,
                'P_NRSEQRETLOTE'    => $nrseqretlote
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updtAtuLotePerda($nrloteestq, $nrsublote, $cdfilial, $nrretest, $nrseqretprod){
        
        try {
            
            $sql = "UPDATE PERDAOP
                     SET NRLOTEESTQ   = :P_NRLOTEESTQ,
                         NRSUBLOTE    = :P_NRSUBLOTE
                   WHERE CDFILIAL     = :P_CDFILIAL
                     AND NRRETEST     = :P_NRRETEST
                     AND NRSEQRETPROD = :P_NRSEQRETPROD";
                
                
                      
            $params = array(
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote,
                'P_CDFILIAL'        => $cdfilial,
                'P_NRRETEST'        => $nrretest,
                'P_NRSEQRETPROD'    => $nrseqretprod
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqParamEstoque(){
        try {
            $sql = "SELECT
                     IDLANCDTMAI,
                     IDLANCDTMEN,
                     NRDIASENTR,
                     NRDIASENTRPD
                  FROM
                     PARAMESTOQUE";
                    

            $params = array();
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqGrid($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT DISTINCT
                     SUBPRETAPAOP.CDFILIAL          ,
                     SUBPRETAPAOP.NRORDPROD         ,
                     SUBPRETAPAOP.NRSUBORDPROD      ,
                     (SUBPRETAPAOP.CDSUBPRODETA) AS CDPRODUTO,
                     SUBPRETAPAOP.NRSUBPRETAOP      ,
                     SUBPRETAPAOP.NRLOTEESTQ        ,
                     SUBPRETAPAOP.NRSUBLOTE         ,
                     PRODUTO.CDARVPROD              ,
                     PRODUTO.NMPRODUTO              ,
                     PRODUTO.SGUNIDADE              ,
                     PRODUTO.VRFATOCONVPR           ,
                     PRODUTO.NRDIASVALANA           ,
                     LOTE.QTPRODLOTE                ,
                     LOTE.DTFABRPROD                ,
                     LOTE.DTVALANALOTE              ,
                     LOTE.IDSTANALISE               ,
                     CUSTOTOT.CUSTOTOTAL CUSTOTOTAL ,
                  
                       ROUND(CUSTOTOT.CUSTOTOTAL/DECODE(CUSTOLOTE.QTTOTLOTE,0,1,CUSTOLOTE.QTTOTLOTE ),5) AS CUSTOUNIT,
                  
                       ROUND((CUSTOTOT.CUSTOTOTAL/DECODE(CUSTOLOTE.QTTOTLOTE,0,1,CUSTOLOTE.QTTOTLOTE )) * LOTE.QTPRODLOTE, 5) CUSTTOTLOTE
                  FROM
                     LOTE, PRODUTO,
                     SUBPRETAPAOP,
                     (SELECT
                         APRCUSTOP.CDFILIAL,
                         APRCUSTOP.NRORDPROD,
                         APRCUSTOP.NRSUBORDPROD,
                         APRCUSTOP.CDPRODUTO,
                         SUM(DECODE(APRCUSTOP.CDFUNCUSTO,
                                     '002', APRCUSTOP.VRAPRCUSTOP * APRCUSTOP.QTHRAPRCUST * QTAPRCUSTOP,
                                     '003', APRCUSTOP.VRAPRCUSTOP * APRCUSTOP.QTHRAPRCUST * QTAPRCUSTOP,
                                            APRCUSTOP.QTAPRCUSTOP * APRCUSTOP.VRAPRCUSTOP)) - NVL(SUM(DECODE(APRCUSTOP.CDFUNCUSTO,
                                     '099', APRCUSTOP.QTAPRCUSTOP * APRCUSTOP.VRAPRCUSTOP)),0)+( SELECT NVL(SUM(VRCUSTERCOP),0) TOTALVRCUSTERCOP
                                        FROM CUSTERCOP
                                     WHERE CDFILIAL     = :P_CDFILIAL
                                       AND NRORDPROD    = :P_NRORDPROD
                                       AND NRSUBORDPROD = :P_NRSUBORDPROD) AS CUSTOTOTAL
                      FROM
                         CUSTOINDI, APRCUSTOP
                      WHERE
                         APRCUSTOP.CDFILIAL     = :P_CDFILIAL           AND
                         APRCUSTOP.NRORDPROD    = :P_NRORDPROD          AND
                         APRCUSTOP.NRSUBORDPROD = :P_NRSUBORDPROD       AND
                         APRCUSTOP.CDFUNCUSTO   = CUSTOINDI.CDFUNCUSTO  AND
                         APRCUSTOP.CDCUSTOINDI  = CUSTOINDI.CDCUSTOINDI AND
                         CUSTOINDI.IDCOMPCUSTPR = 'S'
                        GROUP BY APRCUSTOP.CDFILIAL, APRCUSTOP.NRORDPROD, APRCUSTOP.
                    NRSUBORDPROD, APRCUSTOP.CDPRODUTO
                     ) CUSTOTOT,
                     (SELECT
                         QTLOTE.CDFILIAL,
                         QTLOTE.NRLOTEESTQ,
                         QTLOTE.CDPRODUTO,
                         SUM(QTLOTE.QTPRODLOTE) QTTOTLOTE
                      FROM
                         (SELECT DISTINCT
                             LOTE.CDFILIAL,
                             LOTE.NRLOTEESTQ,
                             LOTE.NRSUBLOTE,
                             LOTE.CDPRODUTO,
                             ROUND(nvl(LOTE.QTPRODLOTE,1), 5) AS QTPRODLOTE
                          FROM
                             LOTE, SUBPRETAPAOP
                          WHERE
                             SUBPRETAPAOP.CDFILIAL     = :P_CDFILIAL        AND
                             SUBPRETAPAOP.NRORDPROD    = :P_NRORDPROD       AND
                             SUBPRETAPAOP.NRSUBORDPROD = :P_NRSUBORDPROD    AND
                             SUBPRETAPAOP.CDFILIAL     = LOTE.CDFILIAL      AND
                             SUBPRETAPAOP.CDSUBPRODETA = LOTE.CDPRODUTO     AND
                             SUBPRETAPAOP.NRLOTEESTQ   = LOTE.NRLOTEESTQ    AND
                             SUBPRETAPAOP.NRSUBLOTE    = LOTE.NRSUBLOTE
                         ) QTLOTE
                      GROUP BY
                         QTLOTE.CDFILIAL,
                         QTLOTE.NRLOTEESTQ,
                         QTLOTE.CDPRODUTO
                     ) CUSTOLOTE
                  WHERE
                     SUBPRETAPAOP.CDFILIAL     = :P_CDFILIAL                  AND
                     SUBPRETAPAOP.NRORDPROD    = :P_NRORDPROD                 AND
                     SUBPRETAPAOP.NRSUBORDPROD = :P_NRSUBORDPROD              AND
                     SUBPRETAPAOP.CDSUBPRODETA = PRODUTO.CDPRODUTO            AND
                     SUBPRETAPAOP.CDFILIAL     = LOTE.CDFILIAL                AND
                     SUBPRETAPAOP.CDSUBPRODETA = LOTE.CDPRODUTO               AND
                     SUBPRETAPAOP.NRLOTEESTQ   = LOTE.NRLOTEESTQ              AND
                     SUBPRETAPAOP.NRSUBLOTE    = LOTE.NRSUBLOTE               AND
                     SUBPRETAPAOP.CDFILIAL     = CUSTOTOT.CDFILIAL  (+)       AND
                     SUBPRETAPAOP.NRORDPROD    = CUSTOTOT.NRORDPROD (+)       AND
                     SUBPRETAPAOP.NRSUBORDPROD = CUSTOTOT.NRSUBORDPROD (+)    AND
                     SUBPRETAPAOP.CDFILIAL     = CUSTOLOTE.CDFILIAL           AND
                     SUBPRETAPAOP.CDSUBPRODETA = CUSTOLOTE.CDPRODUTO          AND
                     SUBPRETAPAOP.NRLOTEESTQ   = CUSTOLOTE.NRLOTEESTQ
                  ORDER BY
                     SUBPRETAPAOP.NRLOTEESTQ,
                     SUBPRETAPAOP.NRSUBLOTE";
                     
                     
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqGridCons($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT DISTINCT
                     SUBPRETAPAOP.CDFILIAL ,
                     SUBPRETAPAOP.NRORDPROD ,
                     SUBPRETAPAOP.NRSUBORDPROD ,
                     (SUBPRETAPAOP.CDSUBPRODETA) AS CDPRODUTO,
                     SUBPRETAPAOP.NRSUBPRETAOP ,
                     SUBPRETAPAOP.NRLOTEESTQ ,
                     SUBPRETAPAOP.NRSUBLOTE ,
                     PRODUTO.CDARVPROD ,
                     PRODUTO.NMPRODUTO ,
                     PRODUTO.SGUNIDADE ,
                     PRODUTO.VRFATOCONVPR ,
                     PRODUTO.NRDIASVALANA ,
                     LOTE.QTPRODLOTE ,
                     LOTE.DTFABRPROD ,
                     LOTE.DTVALANALOTE ,
                     LOTE.IDSTANALISE ,
                     ROUND(CUSTOTOT.CUSTOTOTAL,5) CUSTOTOTAL ,
                  
                       ROUND(CUSTOTOT.CUSTOTOTAL/CUSTOLOTE.QTTOTLOTE, 5) AS CUSTOUNIT,
                  
                       ROUND((CUSTOTOT.CUSTOTOTAL/CUSTOLOTE.QTTOTLOTE) * LOTE.QTPRODLOTE, 5)CUSTTOTLOTE
                  FROM
                  LOTE, PRODUTO,
                  SUBPRETAPAOP,
                  (SELECT
                      APRCUSTOP.CDFILIAL,
                      APRCUSTOP.NRORDPROD,
                      APRCUSTOP.NRSUBORDPROD,
                      APRCUSTOP.CDPRODUTO,
                      SUM(DECODE(APRCUSTOP.CDFUNCUSTO,
                      '002', APRCUSTOP.VRAPRCUSTOP * APRCUSTOP.QTHRAPRCUST,
                      '003', APRCUSTOP.VRAPRCUSTOP * APRCUSTOP.QTHRAPRCUST,
                      APRCUSTOP.QTAPRCUSTOP * APRCUSTOP.VRAPRCUSTOP)) -
                      NVL(SUM(DECODE(APRCUSTOP.CDFUNCUSTO, '099', APRCUSTOP.QTAPRCUSTOP * APRCUSTOP.VRAPRCUSTOP)),0) AS CUSTOTOTAL
                   FROM
                      CUSTOINDI, APRCUSTOP
                   WHERE
                      APRCUSTOP.CDFILIAL = :P_CDFILIAL AND
                      APRCUSTOP.NRORDPROD = :P_NRORDPROD AND
                      APRCUSTOP.NRSUBORDPROD = :P_NRSUBORDPROD AND
                      APRCUSTOP.CDFUNCUSTO = CUSTOINDI.CDFUNCUSTO AND
                      APRCUSTOP.CDCUSTOINDI = CUSTOINDI.CDCUSTOINDI AND
                      CUSTOINDI.IDCOMPCUSTPR = 'S'
                   GROUP BY
                      APRCUSTOP.CDFILIAL,
                      APRCUSTOP.NRORDPROD,
                      APRCUSTOP.NRSUBORDPROD,
                      APRCUSTOP.CDPRODUTO
                  ) CUSTOTOT,
                  
                  (SELECT
                     QTLOTE.CDFILIAL,
                     QTLOTE.NRLOTEESTQ,
                     QTLOTE.CDPRODUTO,
                     SUM(QTLOTE.QTPRODLOTE) QTTOTLOTE
                  FROM
                  (SELECT DISTINCT
                       LOTE.CDFILIAL,
                       LOTE.NRLOTEESTQ,
                       LOTE.NRSUBLOTE,
                       LOTE.CDPRODUTO,
                       LOTE.QTPRODLOTE AS QTPRODLOTE
                       FROM
                       LOTE, SUBPRETAPAOP
                       WHERE
                       SUBPRETAPAOP.CDFILIAL = :P_CDFILIAL AND
                       SUBPRETAPAOP.NRORDPROD = :P_NRORDPROD AND
                       SUBPRETAPAOP.NRSUBORDPROD = :P_NRSUBORDPROD AND
                       SUBPRETAPAOP.CDFILIAL = LOTE.CDFILIAL AND
                       SUBPRETAPAOP.CDSUBPRODETA = LOTE.CDPRODUTO AND
                       SUBPRETAPAOP.NRLOTEESTQ = LOTE.NRLOTEESTQ AND
                       SUBPRETAPAOP.NRSUBLOTE = LOTE.NRSUBLOTE
                       ) QTLOTE
                       GROUP BY
                       QTLOTE.CDFILIAL,
                       QTLOTE.NRLOTEESTQ,
                       QTLOTE.CDPRODUTO
                  ) CUSTOLOTE
                  WHERE
                     SUBPRETAPAOP.CDFILIAL = :P_CDFILIAL AND
                     SUBPRETAPAOP.NRORDPROD = :P_NRORDPROD AND
                     SUBPRETAPAOP.NRSUBORDPROD = :P_NRSUBORDPROD AND
                     SUBPRETAPAOP.CDSUBPRODETA = PRODUTO.CDPRODUTO AND
                     SUBPRETAPAOP.CDFILIAL = LOTE.CDFILIAL AND
                     SUBPRETAPAOP.CDSUBPRODETA = LOTE.CDPRODUTO AND
                     SUBPRETAPAOP.NRLOTEESTQ = LOTE.NRLOTEESTQ AND
                     SUBPRETAPAOP.NRSUBLOTE = LOTE.NRSUBLOTE AND
                     SUBPRETAPAOP.CDFILIAL = CUSTOTOT.CDFILIAL (+) AND
                     SUBPRETAPAOP.NRORDPROD = CUSTOTOT.NRORDPROD (+) AND
                     SUBPRETAPAOP.NRSUBORDPROD = CUSTOTOT.NRSUBORDPROD (+) AND
                     SUBPRETAPAOP.CDSUBPRODETA = CUSTOTOT.CDPRODUTO(+) AND
                     SUBPRETAPAOP.CDFILIAL = CUSTOLOTE.CDFILIAL AND
                     SUBPRETAPAOP.CDSUBPRODETA = CUSTOLOTE.CDPRODUTO AND
                     SUBPRETAPAOP.NRLOTEESTQ = CUSTOLOTE.NRLOTEESTQ
                  ORDER BY
                     SUBPRETAPAOP.NRLOTEESTQ,
                     SUBPRETAPAOP.NRSUBLOTE";
                     
            
                     
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdAcab($cdfilial, $nrtipocusto, $nrordprod, $nrsubordprod, $cdsubprodeta, $nrsubpretaop){
        try {
            $sql = "SELECT
                     NVL(PA.IDATUPRODACA, 'N') AS IDATUPRODACA,
                     PA.CDFILIAL,
                     PA.NRORDPROD,
                     PA.NRSUBORDPROD,
                     PA.CDSUBPRODETA,
                     PA.NRSUBPRETAOP,
                     PA.CDPRODUTO,
                     PA.NRPRODACAB,
                     PROD.CDARVPROD,
                     PROD.NMPRODUTO,
                     NVL(PROD.VRFATOCONVPR, 1) AS VRFATOCONVPR,
                     PROD.SGUNIDADE,
                     PROD.NRDIASVALANA,
                     PA.NRLOTEESTQ,
                     PA.NRSUBLOTE,
                     PA.CDALMOXARIFE,
                     A.DSALMOXARIFE,
                     PA.CDLOCALESTOQ,
                     L.DSLOCALESTOQ,
                     NVL(PA.QTPRACABADO, LO.QTPRODLOTE)  AS QTPRACABADO,
                     PA.VRPRACABADO,
                       NVL(IT.VRUNILANCTO, ROUND(CUSTO_PRODUTO(:P_CDFILIAL, PA.CDPRODUTO, :P_TIPOCUSTO, SYSDATE, SYSDATE), 5)) AS VRCUSTESTOQ,
                     PA.NRLANCESTQ,
                     PA.NRSEQUITEM,
                     LO.VRFATOCONV, PR.IDUTVRFTCONVVAR,
                     (SELECT SUM(
                         SUM(DECODE(APRCUSTOP.CDFUNCUSTO,
                                     '002', APRCUSTOP.VRAPRCUSTOP * APRCUSTOP.QTHRAPRCUST * APRCUSTOP.QTAPRCUSTOP,
                                     '003', APRCUSTOP.VRAPRCUSTOP * APRCUSTOP.QTHRAPRCUST * APRCUSTOP.QTAPRCUSTOP,
                                            APRCUSTOP.QTAPRCUSTOP * APRCUSTOP.VRAPRCUSTOP)) - NVL(SUM(DECODE(APRCUSTOP.CDFUNCUSTO,
                                     '099', APRCUSTOP.QTAPRCUSTOP * APRCUSTOP.VRAPRCUSTOP)),0)+( SELECT NVL(SUM(VRCUSTERCOP),0) TOTALVRCUSTERCOP
                                      FROM CUSTERCOP
                                     WHERE CDFILIAL     = :P_CDFILIAL
                                       AND NRORDPROD    = :P_NRORDPROD
                                       AND NRSUBORDPROD = :P_NRSUBORDPROD)) AS CUSTOTOTAL
                      FROM
                         CUSTOINDI, APRCUSTOP
                      WHERE
                         APRCUSTOP.CDFILIAL     = :P_CDFILIAL           AND
                         APRCUSTOP.NRORDPROD    = :P_NRORDPROD          AND
                         APRCUSTOP.NRSUBORDPROD = :P_NRSUBORDPROD       AND
                         APRCUSTOP.CDFUNCUSTO   = CUSTOINDI.CDFUNCUSTO  AND
                         APRCUSTOP.CDCUSTOINDI  = CUSTOINDI.CDCUSTOINDI AND
                         CUSTOINDI.IDCOMPCUSTPR = 'S'
                        GROUP BY APRCUSTOP.CDFILIAL, APRCUSTOP.NRORDPROD, APRCUSTOP.NRSUBORDPROD, APRCUSTOP.CDPRODUTO
                     ) CUSTOTOTAL
                  FROM
                     ITLANCTOEST IT,
                     PRODACABADO PA,
                     LOCALESTOQUE L,
                     ALMOXARIFADO A,
                     PRODFILI PR,
                     PRODUTO PROD,
                     LOTE LO
                  WHERE
                     (PA.CDFILIAL     = :P_CDFILIAL)         AND
                     (PA.NRORDPROD    = :P_NRORDPROD)        AND
                     (PA.NRSUBORDPROD = :P_NRSUBORDPROD)     AND
                     (PA.CDSUBPRODETA = :P_CDSUBPRODETA)     AND
                     (PA.NRSUBPRETAOP = :P_NRSUBPRETAOP)     AND
                     (PA.CDPRODUTO    = PROD.CDPRODUTO)      AND
                     (LO.CDFILIAL     = PR.CDFILIAL (+))     AND
                     (LO.CDPRODUTO    = PR.CDPRODUTO (+))    AND
                     (PA.CDFILIAL     = A.CDFILIAL (+))      AND
                     (PA.CDALMOXARIFE = A.CDALMOXARIFE (+))  AND
                     (PA.CDFILIAL     = L.CDFILIAL (+))      AND
                     (PA.CDALMOXARIFE = L.CDALMOXARIFE (+))  AND
                     (PA.CDLOCALESTOQ = L.CDLOCALESTOQ (+))  AND
                     (PA.CDFILIAL     = IT.CDFILIAL (+))     AND
                     (PA.NRLANCESTQ   = IT.NRLANCESTQ (+))   AND
                     (PA.NRSEQUITEM   = IT.NRSEQUITEM (+))   AND
                     (PA.CDFILIAL     = LO.CDFILIAL  ) AND
                     (PA.NRLOTEESTQ   = LO.NRLOTEESTQ) AND
                     (PA.NRSUBLOTE    = LO.NRSUBLOTE ) AND
                     (PA.CDPRODUTO    = LO.CDPRODUTO)";
                    
            
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_TIPOCUSTO'       => $nrtipocusto,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod,
                'P_CDSUBPRODETA'    => $cdsubprodeta,
                'P_NRSUBPRETAOP'    => $nrsubpretaop
            );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function atuProdAcab($qtpracabado, $cdfilial, $nrordprod, $nrsubordprod, $cdsubprodeta, $nrsubpretaop, $cdproduto, $nrprodacab){
        
        try {
            $sql = "UPDATE PRODACABADO
                      SET 
                         QTPRACABADO = :P_QTPRACABADO
                      WHERE
                         CDFILIAL           = :P_CDFILIAL AND 
                         NRORDPROD   = :P_NRORDPROD AND
                         NRSUBORDPROD = :P_NRSUBORDPROD AND 
                         CDSUBPRODETA = :P_CDSUBPRODETA AND 
                         NRSUBPRETAOP = :P_NRSUBPRETAOP AND 
                         CDPRODUTO = :P_CDPRODUTO AND
                         NRPRODACAB = :P_NRPRODACAB";
       
            $params = array(
                'P_QTPRACABADO'     => $qtpracabado,
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod,
                'P_CDSUBPRODETA'    => $cdsubprodeta,
                'P_NRSUBPRETAOP'    => $nrsubpretaop,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRPRODACAB'      => $nrprodacab 
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqRetirada($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT
                     IT.CDPRODUTO,
                     PROD.CDARVPROD,
                     PROD.NMPRODUTO
                  FROM
                     PRODUTO PROD, ITEMRETIEST IT, RETIRADAESTO R
                  WHERE
                     (R.CDFILIAL     = :P_CDFILIAL)      AND
                     (R.NRORDPROD    = :P_NRORDPROD)     AND
                     (R.NRSUBORDPROD = :P_NRSUBORDPROD)  AND
                     (R.CDFILIAL     = IT.CDFILIAL)      AND
                     (R.NRRETEST     = IT.NRRETEST)      AND
                     (R.NRLANCESTQ   IS NULL) AND
                     ((R.IDAGUARETIRADA IS NULL OR
                      R.IDAGUARETIRADA = 'N'))           AND
                     (IT.CDPRODUTO   = PROD.CDPRODUTO)
                  
                  UNION
                  
                  SELECT
                     IT.CDPRODUTO,
                     PROD.CDARVPROD,
                     PROD.NMPRODUTO
                  FROM
                     PRODUTO PROD, ITEMRETIEST IT, RETIRADAESTO R, ITLANCTOEST I
                  WHERE
                     R.CDFILIAL       = :P_CDFILIAL      AND
                     R.NRORDPROD      = :P_NRORDPROD     AND
                     R.NRSUBORDPROD   = :P_NRSUBORDPROD  AND
                     R.CDFILIAL       = IT.CDFILIAL      AND
                     R.NRRETEST       = IT.NRRETEST      AND
                     NOT R.NRLANCESTQ IS NULL            AND
                     IT.CDPRODUTO     = PROD.CDPRODUTO   AND
                     I.CDFILIAL       = R.CDFILIAL       AND
                     I.NRLANCESTQ     = R.NRLANCESTQ     AND
                     I.IDEVOLESTOQ    = 'N'";

            $params = array(
                    'P_CDFILIAL'     => $cdfilial,
                    'P_NRORDPROD'    => $nrordprod,
                    'P_NRSUBORDPROD' => $nrsubordprod
                );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdSubs($cdmatprima){
        try {
            $sql = "SELECT CDPRODUTO
                    FROM PRODSUBST
                   WHERE CDMATPRIMA = :P_CDMATPRIMA";

            $params = array(
                    'P_CDMATPRIMA'     => $cdmatprima
                );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqRetiradaProd($nrordprod, $cdfilial, $nrsubordprod){
        try {
            $sql = "SELECT DISTINCT PR.CDPRODUTO, PR.CDARVPROD, PR.NMPRODUTO
                      FROM RETIRADAESTO RT, PRODUTO PR, ITEMRETIEST IT, LOTITRETIEST LT
                   WHERE RT.NRRETEST  = IT.NRRETEST
                     AND RT.CDFILIAL  = IT.CDFILIAL
                     AND PR.CDPRODUTO = IT.CDPRODUTO
                     AND LT.CDFILIAL  = IT.CDFILIAL
                     AND LT.CDPRODUTO = IT.CDPRODUTO
                     AND LT.NRRETEST  = RT.NRRETEST
                     AND LT.NRLANCESTQ IS NOT NULL
                     AND RT.NRORDPROD = :P_NRORDPROD
                     AND RT.CDFILIAL = :P_CDFILIAL
                     AND RT.NRSUBORDPROD = :P_NRSUBORDPROD";

            $params = array(
                    'P_NRORDPROD'       => $nrordprod,
                    'P_CDFILIAL'        => $cdfilial,
                    'P_NRSUBORDPROD'    => $nrsubordprod
                );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqAtividade($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT
                    AUDITORIA.CDPRODUTO,
                    AUDITORIA.NRLOTEESTQ,
                    AUDITORIA.NRSUBLOTE,
                    AUDITORIA.IDTPAUDITADO,
                    INSPECAO.CDARVINSPEC,
                    INSPECAO.NMINSPECAO,
                    INSPECAO.IDORIGINSPEC,
                    NVL( (SELECT  IDTPCONCLETAPA
                            FROM ORDPRODUCAO O, ETAFABRIC E
                           WHERE O.CDFILIAL     = :P_CDFILIAL       AND
                                 O.NRORDPROD    = :P_NRORDPROD   AND
                                 O.NRSUBORDPROD = :P_NRSUBORDPROD          AND
                                 E.CDFILIAL    = O.CDFILIAL  AND
                                 E.CDPRODUTO   = O.CDPRODUTO AND
                                 E.CDPROCFABRIC = O.CDPROCFABRIC AND
              
                                   E.CDETAPAPROD  = AUDITORIA.CDETAPAPROD), 'S') AS IDTPCONCLETAPA,
                    AUDITORIA.IDSTATUSAUDI IDPENAND,
                    AUDITORIA.IDSTATUSAUDI IDNOK
                 FROM
                    INSPECAO, AUDITORIA, INSPECAUDIT
                 WHERE
                    AUDITORIA.CDFILIAL     = :P_CDFILIAL              AND
                    AUDITORIA.NRORDPROD    = :P_NRORDPROD        AND
                    AUDITORIA.NRSUBORDPROD = :P_NRSUBORDPROD         AND
                    AUDITORIA.CDFILIAL     = INSPECAUDIT.CDFILIAL    AND
                    AUDITORIA.NRAUDITORIA  = INSPECAUDIT.NRAUDITORIA AND
                    INSPECAUDIT.CDINSPECAO = INSPECAO.CDINSPECAO";

            $params = array(
                    'P_CDFILIAL'        => $cdfilial,
                    'P_NRORDPROD'       => $nrordprod,
                    'P_NRSUBORDPROD'    => $nrsubordprod
                );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqVProdNFechados($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT
                       CDFILIAL
                    FROM
                       PRODACABADO
                    WHERE
                       (CDFILIAL     = :P_CDFILIAL)     AND
                       (NRORDPROD    = :P_NRORDPROD)    AND
                       (NRSUBORDPROD = :P_NRSUBORDPROD) AND
                       (NRLANCESTQ IS NULL)";

            $params = array(
                    'P_CDFILIAL'        => $cdfilial,
                    'P_NRORDPROD'       => $nrordprod,
                    'P_NRSUBORDPROD'    => $nrsubordprod
                );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updtLote($cdfilial, $cdproduto, $nrlote, $nrsublote, $idstprodlote, $dtconcprod, $dtfabrprod, $dtrealiblote,
                                    $cdopeliblote, $idstanalise, $dtvalanalote){
        
        try {
            
            $sql = "UPDATE LOTE SET IDSTPRODLOTE = :P_IDSTPRODLOTE,
                                  DTCONCPROD   = :P_DTCONCPROD,
                                  IDSTANALISE  = :P_IDSTANALISE,
                                  DTVALANALOTE = NVL(:P_DTVALANALOTE, DTVALANALOTE),
                                  DTFABRPROD   = NVL(:P_DTFABRPROD, DTFABRPROD),
                                  DTREALIBLOTE = NVL(:P_DTREALIBLOTE, DTREALIBLOTE),
                                  CDOPELIBLOTE = NVL(:P_CDOPELIBLOTE, CDOPELIBLOTE),
                                  DTENTRLOTE   = NVL(:P_DTFABRPROD, DTFABRPROD)
                  WHERE CDFILIAL   = :P_CDFILIAL  AND
                        CDPRODUTO  = :P_CDPRODUTO AND
                        NRLOTEESTQ = :P_NRLOTE    AND
                        NRSUBLOTE  = :P_NRSUBLOTE";
                     
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRLOTE'          => $nrlote,
                'P_NRSUBLOTE'       => $nrsublote,
                'P_IDSTPRODLOTE'    => $idstprodlote,
                'P_DTCONCPROD'      => $dtconcprod,
                'P_DTFABRPROD'      => $dtfabrprod,
                'P_DTREALIBLOTE'    => $dtrealiblote,
                'P_CDOPELIBLOTE'    => $cdopeliblote,
                'P_IDSTANALISE'     => $idstanalise,
                'P_DTVALANALOTE'    => $dtvalanalote
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqInsLote($cdfilial, $nrloteestq, $nrsublote, $cdproduto){
        try {
            $sql = "INSERT INTO LOTE
                    (CDFILIAL, NRLOTEESTQ, NRSUBLOTE, CDPRODUTO, IDSTATLOTE, DTENTRLOTE)
                    (SELECT :P_CDFILIAL CDFILIAL, :P_NRLOTEESTQ NRLOTEESTQ, :P_NRSUBLOTE NRSUBLOTE, :P_CDPRODUTO CDPRODUTO,
                  'L' IDSTATLOTE, SYSDATE DTENTRLOTE
                    FROM PARAMGERAL
                   MINUS
                     SELECT CDFILIAL, NRLOTEESTQ,  NRSUBLOTE, :P_CDPRODUTO CDPRODUTO, 'L', SYSDATE
                   FROM LOTE
                   WHERE  :P_CDFILIAL   =  CDFILIAL     AND
                          :P_NRLOTEESTQ =  NRLOTEESTQ   AND
                          :P_NRSUBLOTE  =  NRSUBLOTE    AND
                          :P_CDPRODUTO  =  CDPRODUTO)";

            $params = array(
                    'P_CDFILIAL'     => $cdfilial,
                    'P_NRLOTEESTQ'   => $nrloteestq,
                    'P_NRSUBLOTE'    => $nrsublote,
                    'P_CDPRODUTO'    => $cdproduto
                );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqItLanEst($cdfilial, $nrlancestq){
        try {
            $sql = "SELECT CDFILIMOVI, CDPRODMOVI, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE
                    FROM ITLANCTOEST
                   WHERE CDFILIAL = :P_CDFILIAL
                     AND NRLANCESTQ = :P_NRLANCESTQ";
                    
            $params = array(
                    'P_CDFILIAL'     => $cdmatprima,
                    'P_NRLANCESTQ'   => $nrlancestq
                );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updtProdAcab($cdfilial,
                                $nrordprod,
                                $nrsubordprod,
                                $cdsubprodeta,
                                $nrsubpretaop,
                                $cdproduto,
                                $nrprodacab,
                                $nrlancestq,
                                $nrsequitem){
        
        try {
            
            $sql = "UPDATE PRODACABADO
                    SET NRLANCESTQ = :P_NRLANCESTQ,
                    NRSEQUITEM = :P_NRSEQUITEM
                    WHERE   (CDFILIAL     = :P_CDFILIAL)     AND
                            (NRORDPROD    = :P_NRORDPROD)    AND
                            (NRSUBORDPROD = :P_NRSUBORDPROD) AND
                            (CDSUBPRODETA = :P_CDSUBPRODETA) AND
                            (NRSUBPRETAOP = :P_NRSUBPRETAOP) AND
                            (CDPRODUTO    = :P_CDPRODUTO)    AND
                            (NRPRODACAB   = :P_NRPRODACAB)";
                     
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod,
                'P_CDSUBPRODETA'    => $cdsubprodeta,
                'P_NRSUBPRETAOP'    => $nrsubpretaop,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRPRODACAB'      => $nrprodacab,
                'P_NRLANCESTQ'      => $nrlancestq,
                'P_NRSEQUITEM'      => $nrsequitem
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdOrdProdParc($idfechaopparc, $cdfilial, $nrordprod, $nrsubordprod){
        
        try {
            
            $sql = "UPDATE ORDPRODUCAO
                     SET IDFECHAOPPARC = :P_IDFECHAOPPARC
                   WHERE CDFILIAL = :P_CDFILIAL
                     AND NRORDPROD = :P_NRORDPROD
                     AND NRSUBORDPROD = :P_NRSUBORDPROD";
                     
                      
                     
                    
                     
            $params = array(
                'P_IDFECHAOPPARC'   => $idfechaopparc,
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updtQtLote($cdfilial, $nrloteestq, $nrsublote, $cdproduto, $qtprodlote){
        
        try {
            
            $sql = "UPDATE LOTE
                     SET QTPRODLOTE = :P_QTPRODLOTE
                  WHERE
                      CDFILIAL = :P_CDFILIAL AND
                      NRLOTEESTQ = :P_NRLOTEESTQ AND
                      NRSUBLOTE = :P_NRSUBLOTE AND
                      CDPRODUTO = :P_CDPRODUTO";
                
            $params = array(
                'P_QTPRODLOTE'  => $qtprodlote,
                'P_CDFILIAL'    => $cdfilial,
                'P_NRLOTEESTQ'  => $nrloteestq,
                'P_NRSUBLOTE'   => $nrsublote,
                'P_CDPRODUTO'   => $cdproduto
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqOrdProdPed($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT O.CDFILIAL, O.NRORDPROD, O.NRSUBORDPROD, O.NRPEDIDO, O.NRITEMPED, 
                   P.IDUTRESELOTE, I.QTITEMPED, I.CDPRODUTO, R.NRRESPRODPED
                      FROM ORDPRODUCAO O, PARAMPEDIDO P, RESERPRODPED R, ITEMPEDIDO I
                   WHERE (O.CDFILIAL     = :P_CDFILIAL    )
                     AND (O.NRORDPROD    = :P_NRORDPROD   )
                     AND (O.NRSUBORDPROD = :P_NRSUBORDPROD)
                     AND (O.CDFILIAL     = P.CDFILIAL     )
                     AND (O.CDFILIAL     = I.CDFILIAL     )
                     AND (O.NRPEDIDO     = I.NRPEDIDO     )
                     AND (O.NRITEMPED    = I.NRITEMPED    )   
                     AND (O.CDFILIAL     = R.CDFILIAL  (+))
                     AND (O.NRPEDIDO     = R.NRPEDIDO  (+))
                     AND (O.NRITEMPED    = R.NRITEMPED (+))
                     AND (I.IDSITITEMPED        IN (1,2,3))
                   ORDER BY O.CDFILIAL, O.NRORDPROD, O.NRSUBORDPROD";
                     
            $params = array(
                    'P_CDFILIAL'     => $cdfilial,
                    'P_NRORDPROD'    => $nrordprod,
                    'P_NRSUBORDPROD' => $nrsubordprod
                );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdReserProdPed($cdfilial, $nrpedido, $nritemped, $cdalmoxarife, $cdlocalestoq){
        try {
            
            $sql = "UPDATE RESERPRODPED
                      SET CDALMOXARIFE = :P_CDALMOXARIFE,
                          CDLOCALESTOQ = :P_CDLOCALESTOQ
                    WHERE (CDFILIAL     = :P_CDFILIAL )
                      AND (NRPEDIDO     = :P_NRPEDIDO )
                      AND (NRITEMPED    = :P_NRITEMPED)";
        
            $params = array(
                'P_CDALMOXARIFE'  => $cdalmoxarife,
                'P_CDLOCALESTOQ'  => $cdlocalestoq,
                'P_CDFILIAL'      => $cdfilial,
                'P_NRPEDIDO'      => $nrpedido,
                'P_NRITEMPED'     => $nritemped
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsReserProdPed($cdfilial, $nrpedido, $nritemped, $qtresprodped, $cdproduto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $nrresprodped){
        try {
            
            $sql = "INSERT INTO RESERPRODPED (CDFILIAL, NRRESPRODPED, CDPRODUTO, CDALMOXARIFE, CDLOCALESTOQ,
                                              NRLOTEESTQ, NRSUBLOTE, QTRESPRODPED, NRPEDIDO, NRITEMPED)
                    VALUES (:P_CDFILIAL, :P_NRRESPRODPED, :P_CDPRODUTO, :P_CDALMOXARIFE, :P_CDLOCALESTOQ,
                            :P_NRLOTEESTQ, :P_NRSUBLOTE, :P_QTRESPRODPED, :P_NRPEDIDO, :P_NRITEMPED)";
            
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRRESPRODPED'    => $nrresprodped,
                'P_CDPRODUTO'       => $cdproduto,
                'P_CDALMOXARIFE'    => $cdalmoxarife,
                'P_CDLOCALESTOQ'    => $cdlocalestoq,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote,
                'P_QTRESPRODPED'    => $qtresprodped,
                'P_NRPEDIDO'        => $nrpedido,
                'P_NRITEMPED'       => $nritemped
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdItemPed($cdfilial, $nrpedido, $nritemped){
        try {
            
            $sql = "UPDATE ITEMPEDIDO
                     SET IDITPEDRESER = '1'
                    WHERE (CDFILIAL   = :P_CDFILIAL )
                      AND (NRPEDIDO   = :P_NRPEDIDO )
                      AND (NRITEMPED  = :P_NRITEMPED)";

            $params = array(
                'P_CDFILIAL'    => $cdfilial,
                'P_NRPEDIDO'    => $nrpedido,
                'P_NRITEMPED'   => $nritemped
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqExisteItemEst($cdfilial, $nrlancestq){
        try {
            $sql = "SELECT COUNT(1) AS NRCOUNT
                    FROM ITLANCTOEST
                   WHERE CDFILIAL = :P_CDFILIAL
                     AND NRLANCESTQ = :P_NRLANCESTQ";

            $params = array(
                    'P_CDFILIAL'     => $cdfilial,
                    'P_NRLANCESTQ'   => $nrlancestq
                );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmDelLanctoEst($cdfilial, $nrlancestq){
        try {
            
            $sql = "DELETE FROM LANCTOESTOQ
                      WHERE CDFILIAL = :P_CDFILIAL
                        AND NRLANCESTQ = :P_NRLANCESTQ";

            $params = array(
                'P_CDFILIAL'    => $cdfilial,
                'P_NRLANCESTQ'  => $nrlancestq
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function xcomAtuQTde($cdfilial, $nrloteestq, $nrsublote, $cdproduto, $qtprodlote){
        try {
            
            $sql = "UPDATE LOTE
                     SET QTPRODLOTE = :P_QTPRODLOTE
                  WHERE
                      CDFILIAL = :P_CDFILIAL AND
                      NRLOTEESTQ = :P_NRLOTEESTQ AND
                      NRSUBLOTE = :P_NRSUBLOTE AND
                      CDPRODUTO = :P_CDPRODUTO";

            $params = array(
                'P_QTPRODLOTE'  => $qtprodlote,
                'P_CDFILIAL'    => $cdfilial,
                'P_NRLOTEESTQ'  => $nrloteestq,
                'P_NRSUBLOTE'   => $nrsublote,
                'P_CDPRODUTO'   => $cdproduto
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqVerif_ItemRetiEst($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT
                     LOTITRETIEST.CDFILIAL     ,
                     LOTITRETIEST.NRRETEST     ,
                     LOTITRETIEST.NRSEQRETPROD ,
                     LOTITRETIEST.CDPRODUTO    ,
                     LOTITRETIEST.NRLOTEESTQ   ,
                     LOTITRETIEST.NRSUBLOTE    ,
                     LOTITRETIEST.CDALMOXARIFE ,
                     LOTITRETIEST.CDLOCALESTOQ ,
                     LOTITRETIEST.QTRETPROD    ,
                     LOTITRETIEST.QTDEVRETPROD ,
                     LOTITRETIEST.QTEXTRETPROD ,
                     LOTITRETIEST.QTREJRETPROD ,
                     LOTITRETIEST.QTRETLIQPROD ,
                     LOTITRETIEST.IDTPRETEST   ,
                     LOTITRETIEST.NRSEQRETLOTE ,
                     LOTITRETIEST.DTLANCRETDEV ,
                     ITEMRETIEST.QTPRERETPROD  ,
                     ITEMRETIEST.CDTIPORETI    ,
                     RETIRADAESTO.CDCENTCUST   ,
                     LOTITRETIEST.CDFILIAL AS CDFILIMOVI,
                     LOTITRETIEST.CDPRODUTO AS CDPRODMOVI
                  FROM
                     RETIRADAESTO, ITEMRETIEST, LOTITRETIEST
                  WHERE
                     ITEMRETIEST.CDFILIAL     = LOTITRETIEST.CDFILIAL     AND
                     ITEMRETIEST.NRRETEST     = LOTITRETIEST.NRRETEST     AND
                     ITEMRETIEST.NRSEQRETPROD = LOTITRETIEST.NRSEQRETPROD AND
                     ITEMRETIEST.CDPRODUTO    = LOTITRETIEST.CDPRODUTO    AND
                     ITEMRETIEST.CDFILIAL     = RETIRADAESTO.CDFILIAL     AND
                     ITEMRETIEST.NRRETEST     = RETIRADAESTO.NRRETEST   AND
                     RETIRADAESTO.CDFILIAL    = :P_CDFILIAL AND
                     RETIRADAESTO.NRORDPROD   = :P_NRORDPROD AND
                     RETIRADAESTO.NRSUBORDPROD = :P_NRSUBORDPROD AND
                     RETIRADAESTO.NRLANCESTQ IS NULL
                  ORDER BY
                       LOTITRETIEST.NRRETEST, LOTITRETIEST.NRSEQRETPROD, LOTITRETIES +
                    T.CDPRODUTO";
                   
            $params = array(
                    'P_CDFILIAL'        => $cdfilial,
                    'P_NRORDPROD'       => $nrordprod,
                    'P_NRSUBORDPROD'    => $nrsubordprod
                );
            
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmUp_NrlanEstq($cdfiliallan, $stnrlanctoest, $nrretest, $cdfilial){
        try {
            
            $sql = "UPDATE RETIRADAESTO 
                    SET CDFILIALLAN = :P_CDFILIALLAN,
                        NRLANCESTQ  = :P_NRLANCESTQ
                    WHERE CDFILIAL     = :P_CDFILIAL AND
                        NRRETEST     = :P_NRRETEST";
                      
                

            $params = array(
                'P_CDFILIALLAN' => $cdfiliallan,
                'P_NRLANCESTQ'  => $stnrlanctoest,
                'P_CDFILIAL'    => $cdfilial,
                'P_NRRETEST'    => $nrretest
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
   
    // PREPARA FECHAMENTO
    public function stpAprCustop($row) {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CALCULA_CUSTO_OP');
            $procedure->addParam(new Param('P_CDFILIAL'     , Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRORDPROD'    , Param::PARAM_INPUT, $row['NRORDPROD'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSUBORDPROD' , Param::PARAM_INPUT, $row['NRSUBORDPROD'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO'    , Param::PARAM_INPUT, $row['CDPRODUTO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLOTEESTQ'   , Param::PARAM_INPUT, $row['NRLOTEESTQ'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSUBLOTE'    , Param::PARAM_INPUT, $row['NRSUBLOTE'], Param::PARAM_TYPE_STR));

            $procedure->execute();
        } catch (\Exception $e) {
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
            throw $e;
        }
    }
    
    public function acmDelProdAcab($row){
        try {
            $sql = "DELETE FROM PRODACABADO
                    WHERE (CDFILIAL     = :P_CDFILIAL)     AND
                       (NRORDPROD    = :P_NRORDPROD)    AND
                       (NRSUBORDPROD = :P_NRSUBORDPROD) AND
                       (CDSUBPRODETA = :P_CDSUBPRODETA) AND
                       (NRSUBPRETAOP = :P_NRSUBPRETAOP) AND
                       (CDPRODUTO    = :P_CDPRODUTO)    AND
                       (NRLANCESTQ IS NULL)";
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_NRORDPROD'       => $row['NRORDPROD'],
                'P_NRSUBORDPROD'    => $row['NRSUBORDPROD'],
                'P_CDSUBPRODETA'    => $row['CDPRODUTO'],
                'P_NRSUBPRETAOP'    => $row['NRSUBPRETAOP'],
                'P_CDPRODUTO'       => $row['CDPRODUTO']
            );
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqPesqAlmLocAcab($row){
        try {
            $sql = "SELECT
                       P.CDALMOXPRACA,
                       A.DSALMOXARIFE,
                       P.CDLOCALPRACA,
                       L.DSLOCALESTOQ
                    FROM
                       PRODFILI P,
                       ALMOXARIFADO A,
                       LOCALESTOQUE L
                    WHERE
                       (P.CDFILIAL     = :P_CDFILIAL)        AND
                       (P.CDPRODUTO    = :P_CDPRODUTO)       AND
                       (P.CDFILIAL     = A.CDFILIAL     (+)) AND
                       (P.CDALMOXPRACA = A.CDALMOXARIFE (+)) AND
                       (P.CDFILIAL     = L.CDFILIAL     (+)) AND
                       (P.CDALMOXPRACA = L.CDALMOXARIFE (+)) AND
                       (P.CDLOCALPRACA = L.CDLOCALESTOQ (+))";
                     
            $params = array(
                'P_CDFILIAL'        => $row['CDFILIAL'],
                'P_CDPRODUTO'       => $row['CDPRODUTO']
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function xQryUtSeg($row){
        try {
            $sql = "SELECT  IDUTSEGLIBLOTE
                    FROM PARAMAN
                    WHERE 
                       CDFILIAL = :P_CDFILIAL";
                           
            
            $params = array(
                'P_CDFILIAL'         => $row['CDFILIAL']
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function xqryAlmoxQuar($row){
        try {
            $sql = "SELECT
                       NVL(CDALMOXQUAR, CDALMOXPRACA) AS CDALMOXQUAR,
                       NVL(CDLOCALQUAR, CDLOCALPRACA) AS CDLOCALQUAR
                    FROM
                       PRODFILI
                    WHERE
                       CDFILIAL = :P_CDFILIAL AND
                       CDPRODUTO = :P_CDPRODUTO";
            
            $params = array(
                'P_CDFILIAL'         => $row['CDFILIAL'],
                'P_CDPRODUTO'        => $row['CDPRODUTO']
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdAcabTerc($row){
        try {
            $sql = "SELECT A.CDFILIAL, A.CDCLIENTE, A.CDALMOXARIFE, A.CDLOCALESTOQ, A.CDALMOXTERCCLI, 
                           A.CDLOCALTERCCLI, E.IDUTILALMOXTERC
                      FROM AGENTETERCECLI A,
                           ORDPRODUCAO    O,
                           PEDIDO         P,
                           PAREXPEDICAO   E
                     WHERE (O.CDFILIAL     = :P_CDFILIAL    )
                       AND (O.NRORDPROD    = :P_NRORDPROD   )
                       AND (O.NRSUBORDPROD = :P_NRSUBORDPROD)
                       AND (O.CDFILIAL     = P.CDFILIAL     )
                       AND (O.NRPEDIDO     = P.NRPEDIDO     )
                       AND (A.CDFILIAL     = P.CDFILIAL     )   
                       AND (A.CDCLIENTE    = P.CDCLIENTE    )
                       AND (A.CDFILIAL     = E.CDFILIAL     )   
                       AND (IDPEDINDUSTRIA = 'S'            )
                     ORDER BY CDCLIENTE";
            
            $params = array(
                'P_CDFILIAL'         => $row['CDFILIAL'],
                'P_NRORDPROD'        => $row['NRORDPROD'],
                'P_NRSUBORDPROD'     => $row['NRSUBORDPROD']
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqSeqProdAcab($row){
        try {
            $sql = "SELECT
                        NVL(MAX(NRPRODACAB), '0') NRPRODACAB
                    FROM
                       PRODACABADO
                    WHERE
                       (CDFILIAL     = :P_CDFILIAL)     AND
                       (NRORDPROD    = :P_NRORDPROD)    AND
                       (NRSUBORDPROD = :P_NRSUBORDPROD) AND
                       (CDSUBPRODETA = :P_CDSUBPRODETA) AND
                       (NRSUBPRETAOP = :P_NRSUBPRETAOP)";

            $params = array(
                'P_CDFILIAL'         => $row['CDFILIAL'],
                'P_NRORDPROD'        => $row['NRORDPROD'],
                'P_NRSUBORDPROD'     => $row['NRSUBORDPROD'],
                'P_CDSUBPRODETA'     => $row['CDPRODUTO'],
                'P_NRSUBPRETAOP'     => $row['NRSUBPRETAOP']
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsProdAcab($idatuprodaca,
                            $cdfilial,
                            $nrordprod,
                            $nrsubordprod,
                            $cdsubprodeta,
                            $nrsubpretaop,
                            $cdproduto,
                            $nrprodacab,
                            $nrloteestq,
                            $nrsublote,
                            $stCdAlmox,
                            $stCdLocEst,
                            $custounit){
        try {
            $sql = "INSERT INTO PRODACABADO
                      (IDATUPRODACA, CDFILIAL, NRORDPROD, NRSUBORDPROD, CDSUBPRODETA,
                       NRSUBPRETAOP, CDPRODUTO, NRPRODACAB, CDALMOXARIFE, CDLOCALESTOQ,
                       NRLOTEESTQ, NRSUBLOTE, VRPRACABADO)
                    values
                      (:P_IDATUPRODACA, 
                        :P_CDFILIAL, 
                        :P_NRORDPROD, 
                        :P_NRSUBORDPROD, 
                        :P_CDSUBPRODETA,
                        :P_NRSUBPRETAOP, 
                        :P_CDPRODUTO, 
                        :P_NRPRODACAB, 
                        :P_CDALMOXARIFE, 
                        :P_CDLOCALESTOQ,
                        :P_NRLOTEESTQ, 
                        :P_NRSUBLOTE, 
                        :P_VRPRACABADO)";
            $params = array(
                'P_IDATUPRODACA'    => $idatuprodaca,
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod,
                'P_CDSUBPRODETA'    => $cdsubprodeta,
                'P_NRSUBPRETAOP'    => $nrsubpretaop,
                'P_CDPRODUTO'       => $cdproduto,
                'P_NRPRODACAB'      => $nrprodacab,
                'P_CDALMOXARIFE'    => $stCdAlmox,
                'P_CDLOCALESTOQ'    => $stCdLocEst,
                'P_NRLOTEESTQ'      => $nrloteestq,
                'P_NRSUBLOTE'       => $nrsublote,
                'P_VRPRACABADO'     => $custounit
            );
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdProdAcab($cdfilial, $nrordprod, $nrsubordprod, $cdsubprodeta, $nrsubpretaop, $custounit){
        try {
            $sql = "UPDATE PRODACABADO
                    SET VRPRACABADO = :P_VRPRACABADO
                    WHERE CDFILIAL = :P_CDFILIAL
                    AND NRORDPROD = :P_NRORDPROD
                    AND NRSUBORDPROD = :P_NRSUBORDPROD
                    AND CDSUBPRODETA = :P_CDSUBPRODETA
                    AND NRSUBPRETAOP = :P_NRSUBPRETAOP";
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod,
                'P_CDSUBPRODETA'    => $cdsubprodeta,
                'P_NRSUBPRETAOP'    => $nrsubpretaop,
                'P_VRPRACABADO'     => $custounit
            );
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqMinus($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT PR.CDPRODUTO,
                        PR.CDARVPROD,
                        PR.NMPRODUTO
                    FROM PRODUTO PR,
                        INSUMETAPA IP,
                        ORDPRODUCAO OP,
                        PROCFABRICA PF,
                        PRODFILI PL
                    WHERE PR.CDPRODUTO           = IP.CDPRODINSUMO
                    AND OP.CDPRODUTO             = IP.CDPRODUTO
                    AND OP.CDFILIAL              = IP.CDFILIAL
                    AND OP.CDFILIAL              = PF.CDFILIAL
                    AND OP.CDPROCFABRIC          = PF.CDPROCFABRIC
                    AND IP.CDPROCFABRIC          = PF.CDPROCFABRIC
                    AND OP.CDPRODUTO             = PF.CDPRODUTO
                    AND OP.NRORDPROD             = :P_NRORDPROD
                    AND OP.CDFILIAL              = :P_CDFILIAL
                    AND OP.NRSUBORDPROD          = :P_NRSUBORDPROD
                    AND IP.CDFILIAL              = PL.CDFILIAL(+)
                    AND IP.CDPRODINSUMO          = PL.CDPRODUTO(+)
                    AND NVL(PL.IDCNTRESTOQ, 'S') = 'S'
                    MINUS
                    SELECT DISTINCT PR.CDPRODUTO,
                        PR.CDARVPROD,
                        PR.NMPRODUTO
                    FROM RETIRADAESTO RT,
                        PRODUTO PR,
                        ITEMRETIEST IT,
                        LOTITRETIEST LT,
                        PRODFILI PL
                    WHERE RT.NRRETEST            = IT.NRRETEST
                    AND RT.CDFILIAL              = IT.CDFILIAL
                    AND PR.CDPRODUTO             = IT.CDPRODUTO
                    AND LT.CDFILIAL              = IT.CDFILIAL
                    AND LT.CDPRODUTO             = IT.CDPRODUTO
                    AND LT.NRRETEST              = RT.NRRETEST
                    AND LT.NRLANCESTQ           IS NOT NULL
                    AND RT.NRORDPROD             = :P_NRORDPROD
                    AND RT.CDFILIAL              = :P_CDFILIAL
                    AND RT.NRSUBORDPROD          = :P_NRSUBORDPROD
                    AND LT.CDFILIAL              = PL.CDFILIAL(+)
                    AND LT.CDPRODUTO             = PL.CDPRODUTO(+)
                    AND NVL(PL.IDCNTRESTOQ, 'S') = 'S'";

            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_NRORDPROD'        => $nrordprod,
                'P_NRSUBORDPROD'     => $nrsubordprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getDtProcessa($cdfilial){
        try {
            $sql = "SELECT NVL(DTPROCESSA, SYSDATE) 
                    FROM PARAMFILIAL
                    WHERE CDFILIAL = :P_CDFILIAL";
            $params = array(
                'P_CDFILIAL'         => $cdfilial
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqLote($cdfilial, $nrloteestq, $nrsublote, $cdproduto){
        try {
            $sql = "SELECT
                       NRLOTEESTQ,
                       DTREALIBLOTE,
                       CDOPELIBLOTE
                    FROM
                       LOTE
                    WHERE
                       CDFILIAL   = :P_CDFILIAL AND
                       NRLOTEESTQ = :P_NRLOTEESTQ AND
                       NRSUBLOTE  = :P_NRSUBLOTE  AND
                       CDPRODUTO  = :P_CDPRODUTO";
            $params = array(
                'P_CDFILIAL'         => $cdfilial,
                'P_NRLOTEESTQ'       => $nrloteestq,
                'P_NRSUBLOTE'        => $nrsublote,
                'P_CDPRODUTO'        => $cdproduto
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpValLote($cdfilial, $cdproduto, $stnrloteestq, $stnrsublot){
        try {
            $sql = "SELECT * FROM DUAL";
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_CDPRODUTO'       => $cdproduto,
                'P_STNRLOTEESTQ'    => $stnrloteestq,
                'P_STNRSUBLOT'      => $stnrsublot
            );
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqOpsPendentes2($cdfilial, $nrordprod){
        try {
            $sql = "SELECT NRORDPROD,
                      NRSUBORDPROD,
                      CDPRODUTO,
                      QTAPRODPREOP,
                      IDSITORDPROD
                    FROM ORDPRODUCAO
                    WHERE CDFILIAL    = :P_CDFILIAL
                    AND IDSITORDPROD IN ('1','2')
                    ORDER BY NRSUBORDPROD";
                    
            $params = array(
                'P_CDFILIAL'    => $cdfilial,
                'P_NRORDPROD'   => $nrordprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqQtdeFechamento($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT SUM(P.QTPRACABADO)  AS QTPRACABADO
                        FROM PRODACABADO P, ORDPRODUCAO O
                    WHERE P.CDFILIAL     = :P_CDFILIAL
                        AND P.NRORDPROD    = :P_NRORDPROD
                        AND P.NRSUBORDPROD = :P_NRSUBORDPROD
                        AND P.CDFILIAL     = O.CDFILIAL
                        AND P.NRORDPROD    = O.NRORDPROD
                        AND P.NRSUBORDPROD = O.NRSUBORDPROD
                        AND P.CDPRODUTO    = O.CDPRODUTO";
                    
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmUp_OrdProd($status, $dtfim, $cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "UPDATE ORDPRODUCAO SET IDSITORDPROD = :P_STATUS,
                                           DTHRFIMREAOP = :P_DTFIM
                    WHERE CDFILIAL     = :P_CDFILIAL     AND
                          NRORDPROD    = :P_NRORDPROD    AND
                          NRSUBORDPROD = :P_NRSUBORDPROD";
                          
            $params = array(
                'P_STATUS'          => $status,
                'P_DTFIM'           => $dtfim,
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod
            );
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function adqOrdensFilhas($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "SELECT NRORDPROD, NRSUBORDPROD 
                        FROM ORDPRODUCAO
                    WHERE NRORDPRODCON  = :P_NRORDPROD
                        AND NRSUBOPCON  = :P_NRSUBORDPROD
                        AND CDFILIAL    = :P_CDFILIAL";
                    
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod
            );
            $result = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // $result = isset($result[0])? $result[0] : $result;
            return $result;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpStatusCon($status, $dtfim, $cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "UPDATE ORDPRODUCAO SET IDSITORDPROD    = :P_STATUS,
                                           DTHRFIMREAOP    = :P_DTFIM
                    WHERE CDFILIAL      = :P_CDFILIAL
                    AND NRORDPRODCON    = :P_NRORDPROD
                    AND NRSUBOPCON      = :P_NRSUBORDPROD";
                          
            $params = array(
                'P_STATUS'          => $status,
                'P_DTFIM'           => $dtfim,
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod
            );
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function xcmLiberaEquip($cdfilial, $nrordprod, $nrsubordprod){
        try {
            $sql = "DELETE FROM RESERVEQUIPAPRO
                        WHERE CDFILIAL      = :P_CDFILIAL AND
                            NRORDPROD       = :P_NRORDPROD AND
                            NRSUBORDPROD    = :P_NRSUBORDPROD";
                          
            $params = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRORDPROD'       => $nrordprod,
                'P_NRSUBORDPROD'    => $nrsubordprod
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insertTesteConexao($flag){
        try {
            
            $sql = "INSERT INTO TABELA_TESTE_CONEXAO (FLAG)
                    VALUES (:P_FLAG)";
            
            $params = array(
                'P_FLAG'        => $flag
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
}

?>