<?php
namespace Teknisa\CDP\Modules\Producao\Factories;

use Doctrine\ORM\EntityManager;

class cdpExtratoOrdemProducaoFactory {
    
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }
    
    public function buscaDadosHeader($cdfilial,$nrordprod,$nrsubop){

        $sql = "SELECT
                   O.NRORDPROD , O.NRSUBORDPROD, O.NRLOTEESTQ  , P.NMPRODUTO , P.CDARVPROD ,
                   L.DTVALILOTE, O.QTAPRODPREOP, O.DTHRINIREAOP, O.DTGERORDPROD,
                   O.CDFILIAL  , O.CDPROCFABRIC, O.CDPRODUTO, O.DTHRINICIAOP
                FROM
                   ORDPRODUCAO O, PRODUTO P, LOTE L
                WHERE (O.CDFILIAL     = :CDFILIAL )   
                  AND (O.NRORDPROD    = :NRORDPROD )   
                  AND (O.NRSUBORDPROD = NVL(:NRSUBORDPROD   , O.NRSUBORDPROD)) 
                  AND (O.CDPRODUTO = P.CDPRODUTO)
                  AND (O.NRLOTEESTQ = L.NRLOTEESTQ)
                  AND (O.NRSUBLOTE = L.NRSUBLOTE)
                ORDER BY
                   O.NRORDPROD, O.NRSUBORDPROD, P.NMPRODUTO, O.CDPROCFABRIC";
        
        $params = array("CDFILIAL" => $cdfilial, "NRORDPROD" => $nrordprod, "NRSUBORDPROD" => $nrsubop);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function buscaListaEtapa($cdfilial,$nrordprod,$nrsubop) {
        
        $sql = "SELECT
                    ET.CDETAPAPROD, EP.NMETAPAPROD, ET.CDPRODUTO, EO.CDFILIAL, 
                    EO.NRORDPROD, EO.NRSUBORDPROD
                FROM
                    ORDPRODUCAO O, ETAFABRICOP EO, ETAFABRIC ET, ETAPAPROD EP
                WHERE   (O.CDFILIAL     = :CDFILIAL       ) 
                    AND (O.NRORDPROD    = :NRORDPROD      )
                    AND (O.NRSUBORDPROD = :NRSUBORDPROD   )
                    AND (O.CDFILIAL     = EO.CDFILIAL     ) 
                    AND (O.NRORDPROD    = EO.NRORDPROD    )
                    AND (O.NRSUBORDPROD = EO.NRSUBORDPROD )
                    AND (ET.CDFILIAL     = EO.CDFILIAL    )
                    AND (ET.CDPRODUTO    = O.CDPRODUTO    )
                    AND (ET.CDPROCFABRIC = O.CDPROCFABRIC )
                    AND (EO.CDETAPAPROD  = ET.CDETAPAPROD )
                    AND (ET.CDETAPAPROD  = EP.CDETAPAPROD )";
                    
        $params = array("CDFILIAL" => $cdfilial, "NRORDPROD" => $nrordprod, "NRSUBORDPROD" => $nrsubop);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function buscaListaInsumo($cdfilial,$nrordprod,$nrsubop,$cdetapaprod){
        
        $sql = "SELECT 
                    L.NRLOTEESTQ, L.QTREJRETPROD AS QTDECOR, L.QTRETPROD AS QTDETEOR,
                    LT.VRAPROVPROD, P.CDARVPROD, P.NMPRODUTO, P.SGUNIDADE
                FROM
                    ORDPRODUCAO O, RETIRADAESTO R, ITEMRETIEST I, LOTITRETIEST L, PRODUTO P, LOTE LT, ETAPAPROD EP
                WHERE  (O.CDFILIAL     = :CDFILIAL     ) AND
                    (O.NRORDPROD    = :NRORDPROD       ) AND
                    (O.NRSUBORDPROD = :NRSUBORDPROD    ) AND
                    (EP.CDETAPAPROD = :CDETAPAPROD     ) AND
                    (O.CDFILIAL     = R.CDFILIAL       ) AND
                    (O.NRORDPROD    = R.NRORDPROD      ) AND
                    (O.NRSUBORDPROD = R.NRSUBORDPROD   ) AND
                    (EP.CDETAPAPROD  = R.CDETAPAPROD   ) AND
                    (R.CDFILIAL     = I.CDFILIAL       ) AND
                    (R.NRRETEST     = I.NRRETEST       ) AND
                    (I.CDPRODUTO    = P.CDPRODUTO      ) AND
                    (I.CDFILIAL     = L.CDFILIAL    (+)) AND
                    (I.CDPRODUTO    = L.CDPRODUTO   (+)) AND
                    (I.NRRETEST     = L.NRRETEST    (+)) AND
                    (I.NRSEQRETPROD = L.NRSEQRETPROD(+)) AND
                    (L.CDFILIAL     = LT.CDFILIAL   (+)) AND
                    (L.NRLOTEESTQ   = LT.NRLOTEESTQ (+)) AND
                    (L.CDPRODUTO    = LT.CDPRODUTO  (+))";
                    
        $params = array("CDFILIAL" => $cdfilial, "NRORDPROD" => $nrordprod, "NRSUBORDPROD" => $nrsubop, "CDETAPAPROD" => $cdetapaprod);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function buscaAtividade($cdfilial,$nrordprod,$nrsubop,$cdetapaprod){
        $sql = "SELECT
                   A.NRORDIMPAUD,
                   P.CDARVINSPEC, 
                   P.NMINSPECAO, 
                   D.TXDESCINSPEC_CLOB, 
                   I.CDINSPECAO  ,
                   I.DTINIRINSAUD, 
                   I.DTFINRINSAUD, 
                   OP.NMOPERADOR
                FROM
                   ORDPRODUCAO O, AUDITORIA A, INSPECAUDIT I, OPERADOR OP, INSPECAO P, DESCINSPECAO D, ETAPAPROD EP
                WHERE
                    (O.CDFILIAL     = :CDFILIAL      ) AND
                    (O.NRORDPROD    = :NRORDPROD     ) AND
                    (O.NRSUBORDPROD = :NRSUBORDPROD  ) AND
                    (EP.CDETAPAPROD = :CDETAPAPROD   ) AND
                    (O.CDFILIAL     = A.CDFILIAL      ) AND
                    (O.NRORDPROD    = A.NRORDPROD     ) AND
                    (O.NRSUBORDPROD = A.NRSUBORDPROD  ) AND
                    (EP.CDETAPAPROD  = A.CDETAPAPROD   ) AND
                    (A.CDFILIAL     = I.CDFILIAL     ) AND
                    (A.NRAUDITORIA  = I.NRAUDITORIA  ) AND
                    (I.CDINSPECAO   = P.CDINSPECAO   ) AND
                    (I.CDINSPECAO   = D.CDINSPECAO(+)) AND
                    (I.CDOPERADOR   = OP.CDOPERADOR(+))
                ORDER BY
                   A.NRPRIORAUDI, A.NRORDIMPAUD";
                   
        $params = array("CDFILIAL" => $cdfilial, "NRORDPROD" => $nrordprod, "NRSUBORDPROD" => $nrsubop, "CDETAPAPROD" => $cdetapaprod);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function buscaPerda($cdfilial,$nrordprod,$nrsubop,$cdetapaprod){
        $sql = "SELECT
                    D.QTPERDAOP, T.NMTIPOPERDA, E.NMEQUIPPRO, 
                    P.CDARVPROD, P.NMPRODUTO, D.NRPERDAOP
                FROM
                    ORDPRODUCAO O, PRODUTO P, TIPOPERDA T, EQUIPAPROD E, PERDAOP D, ETAPAPROD EP
                WHERE
                    (O.CDFILIAL     = :CDFILIAL      ) AND
                    (O.NRORDPROD    = :NRORDPROD     ) AND
                    (O.NRSUBORDPROD = :NRSUBORDPROD  ) AND
                    (EP.CDETAPAPROD = :CDETAPAPROD   ) AND
                    (O.CDFILIAL     = D.CDFILIAL       ) AND
                    (O.NRORDPROD    = D.NRORDPROD      ) AND
                    (O.NRSUBORDPROD = D.NRSUBORDPROD   ) AND
                    (EP.CDETAPAPROD  = D.CDETAPAPROD    ) AND
                    (D.CDPRODPERDA  = P.CDPRODUTO     ) AND
                    (D.CDTIPOPERDA  = T.CDTIPOPERDA(+)) AND
                    (D.CDEQUIPPRO   = E.CDEQUIPPRO (+))
                ORDER BY
                    D.CDPRODPERDA, D.CDEQUIPPRO";
                    
        $params = array("CDFILIAL" => $cdfilial, "NRORDPROD" => $nrordprod, "NRSUBORDPROD" => $nrsubop, "CDETAPAPROD" => $cdetapaprod);
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
}